@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align.cpp 662 2007-02-17 00:55:41Z bengardner $
+ * $Id: align.cpp 932 2007-09-25 19:04:17Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -150,7 +150,6 @@ static void align_stack(ChunkStack& cs, int col, bool align_single, log_sev_t se
    cs.Reset();
 }
 
-
 /**
  * Adds an item to the align stack and adjust the nl_count and max_col.
  * Adjust max_col as needed
@@ -198,7 +197,6 @@ static void align_add(ChunkStack& cs, chunk_t *pc, int& max_col, int min_pad)
    }
 }
 
-
 void align_all(void)
 {
    if (cpd.settings[UO_align_typedef_span].n > 0)
@@ -237,7 +235,6 @@ void align_all(void)
    }
 }
 
-
 /**
  * Aligns all function prototypes in the file.
  */
@@ -273,7 +270,6 @@ void align_func_proto(int span)
    align_stack(cs, max_col, false, LALPROTO);
 }
 
-
 /**
  * Aligns all backslah-newline combos in the file.
  * This should be done LAST.
@@ -294,7 +290,6 @@ void align_backslash_newline(void)
    }
 }
 
-
 void align_right_comments(void)
 {
    chunk_t *pc;
@@ -313,7 +308,6 @@ void align_right_comments(void)
    }
 }
 
-
 /**
  * Aligns stuff inside a multi-line "= { ... }" sequence.
  */
@@ -324,7 +318,9 @@ void align_struct_initializers(void)
    pc = chunk_get_head();
    while (pc != NULL)
    {
-      if ((pc->type == CT_BRACE_OPEN) && (pc->parent_type == CT_ASSIGN))
+      if ((pc->parent_type == CT_ASSIGN) &&
+          ((pc->type == CT_BRACE_OPEN) ||
+           ((cpd.lang_flags & LANG_D) && (pc->type == CT_SQUARE_OPEN))))
       {
          align_init_brace(pc);
       }
@@ -332,7 +328,6 @@ void align_struct_initializers(void)
    }
 }
 
-
 /**
  * Scans the whole file for #defines. Aligns all within X lines of each other
  */
@@ -411,7 +406,6 @@ void align_preprocessor(void)
    asf.End();
 }
 
-
 /**
  * Aligns all assignment operators on the same level as first, starting with
  * first.
@@ -551,7 +545,6 @@ chunk_t *align_assign(chunk_t *first, int span, int thresh)
    return(pc);
 }
 
-
 /**
  * Counts how many '*' pointers are in a row, going backwards
  *
@@ -570,7 +563,6 @@ int count_prev_ptr_type(chunk_t *pc)
    return(count);
 }
 
-
 static chunk_t *align_func_param(chunk_t *start)
 {
    AlignStack as;
@@ -579,9 +571,10 @@ static chunk_t *align_func_param(chunk_t *start)
    as.Start(2, 0);
 
    as.m_star_style = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_star_style].n;
+   as.m_amp_style  = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_amp_style].n;
 
    bool did_this_line = false;
-   int  comma_count = 0;
+   int  comma_count   = 0;
 
    while ((pc = chunk_get_next(pc)) != NULL)
    {
@@ -623,7 +616,7 @@ static chunk_t *align_func_param(chunk_t *start)
 
 static void align_func_params()
 {
-   chunk_t    *pc;
+   chunk_t *pc;
 
    pc = chunk_get_head();
    while ((pc = chunk_get_next(pc)) != NULL)
@@ -666,7 +659,8 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    if ((start->parent_type == CT_STRUCT) ||
        (start->parent_type == CT_UNION))
    {
-      myspan = cpd.settings[UO_align_var_struct_span].n;
+      myspan   = cpd.settings[UO_align_var_struct_span].n;
+      mythresh = cpd.settings[UO_align_var_struct_thresh].n;
    }
    else
    {
@@ -692,11 +686,22 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    as.Start(myspan, mythresh);
 
    as.m_star_style = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_star_style].n;
+   as.m_amp_style  = (AlignStack::StarStyle)cpd.settings[UO_align_var_def_amp_style].n;
 
    bool did_this_line = false;
    pc = chunk_get_next_ncnl(start);
    while ((pc != NULL) && ((pc->level >= start->level) || (pc->level == 0)))
    {
+      if (chunk_is_comment(pc))
+      {
+         if (pc->nl_count > 0)
+         {
+            as.NewLines(pc->nl_count);
+         }
+         pc = chunk_get_next(pc);
+         continue;
+      }
+
       /* process nested braces */
       if (pc->type == CT_BRACE_OPEN)
       {
@@ -720,7 +725,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
       /* don't align stuff inside parens/squares/angles */
       if (pc->level > pc->brace_level)
       {
-         pc = chunk_get_next_nc(pc);
+         pc = chunk_get_next(pc);
          continue;
       }
 
@@ -744,7 +749,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
          }
          did_this_line = true;
       }
-      pc = chunk_get_next_nc(pc);
+      pc = chunk_get_next(pc);
    }
 
    as.End();
@@ -752,7 +757,6 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    return(pc);
 }
 
-
 /**
  * For a series of lines ending in backslash-newline, align them.
  * The series ends when a newline or multi-line C comment is encountered.
@@ -794,7 +798,6 @@ chunk_t *align_nl_cont(chunk_t *start)
    return(pc);
 }
 
-
 /**
  * For a series of lines ending in a comment, align them.
  * The series ends when more than align_right_cmt_span newlines are found.
@@ -829,7 +832,6 @@ chunk_t *align_trailing_comments(chunk_t *start)
    return(chunk_get_next(pc));
 }
 
-
 /**
  * Shifts out all columns by a certain amount.
  *
@@ -845,7 +847,6 @@ void ib_shift_out(int idx, int num)
    }
 }
 
-
 /**
  * Scans a line for stuff to align on.
  *
@@ -1165,7 +1166,6 @@ static void align_init_brace(chunk_t *start)
    } while ((pc != NULL) && (pc->level > start->level));
 }
 
-
 /**
  * Aligns simple typedefs that are contained on a single line each.
  * This should be called after the typedef target is marked as a type.
@@ -1179,7 +1179,6 @@ static void align_init_brace(chunk_t *start)
 static void align_typedefs(int span)
 {
    chunk_t    *pc;
-   chunk_t    *next;
    chunk_t    *c_type    = NULL;
    chunk_t    *c_typedef = NULL;
    AlignStack as;
@@ -1187,6 +1186,7 @@ static void align_typedefs(int span)
    as.Start(span);
    as.m_gap        = cpd.settings[UO_align_typedef_gap].n;
    as.m_star_style = (AlignStack::StarStyle)cpd.settings[UO_align_typedef_star_style].n;
+   as.m_amp_style  = (AlignStack::StarStyle)cpd.settings[UO_align_typedef_amp_style].n;
 
    pc = chunk_get_head();
    while (pc != NULL)
@@ -1198,45 +1198,20 @@ static void align_typedefs(int span)
       }
       else if (c_typedef != NULL)
       {
-         /* Already hit a typedef on this line */
-         if (pc->type == CT_TYPE)
+         if (pc->flags & PCF_ANCHOR)
          {
-            c_type = pc;
-         }
-         else if (chunk_is_semicolon(pc))
-         {
-            if ((c_type != NULL) && (c_typedef->orig_line == c_type->orig_line))
-            {
-               if (chunk_get_prev_ncnl(pc) == c_type)
-               {
-                  as.Add(c_type);
-               }
-            }
-            c_type    = NULL;
+            as.Add(pc);
             c_typedef = NULL;
          }
-         else
-         {
-            /* don't care */
-         }
       }
       else
       {
          if (pc->type == CT_TYPEDEF)
          {
-            next = chunk_get_next_ncnl(pc);
-            if ((next != NULL) && (next->type == CT_PAREN_OPEN))
-            {
-               LOG_FMT(LALTD, "%s: line %d, col %d - skip function type\n",
-                       __func__, pc->orig_line, pc->orig_col);
-            }
-            else
-            {
-               LOG_FMT(LALTD, "%s: line %d, col %d\n",
-                       __func__, pc->orig_line, pc->orig_col);
-               c_typedef = pc;
-               c_type    = NULL;
-            }
+            LOG_FMT(LALTD, "%s: line %d, col %d\n",
+                    __func__, pc->orig_line, pc->orig_col);
+            c_typedef = pc;
+            c_type    = NULL;
          }
       }
 
