@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align_stack.h 611 2006-12-20 04:12:46Z bengardner $
+ * $Id: align_stack.h 893 2007-08-29 20:43:15Z bengardner $
  */
 
 #include "ChunkStack.h"
@@ -33,14 +33,15 @@ public:
    int        m_gap;
    bool       m_right_align;
    StarStyle  m_star_style;
+   StarStyle  m_amp_style;
 
 
    AlignStack()
    {
-   };
+   }
    ~AlignStack()
    {
-   };
+   }
 
    void Start(int span, int threshold = 0);
    void Add(chunk_t *pc, int seqnum = 0);
