@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: brace_cleanup.cpp 708 2007-03-06 02:16:16Z bengardner $
+ * $Id: brace_cleanup.cpp 987 2007-11-02 18:53:04Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -66,7 +66,7 @@ static int preproc_start(struct parse_frame *frm, chunk_t *pc)
       else
       {
          /* Check for #if, #else, #endif, etc */
-         pp_level = pf_check(frm, next);
+         pp_level = pf_check(frm, pc);
       }
    }
    return(pp_level);
@@ -97,7 +97,6 @@ static void print_stack(int logsev, const char *str,
    }
 }
 
-
 /**
  * Scans through the whole list and does stuff.
  * It has to do some tricks to parse preprocessors.
@@ -170,6 +169,35 @@ void brace_cleanup(void)
    }
 }
 
+/**
+ * pc is a CT_WHILE.
+ * Scan backwards to see if we find a brace/vbrace with the parent set to CT_DO
+ */
+static bool maybe_while_of_do(chunk_t *pc)
+{
+   chunk_t *prev;
+
+   prev = chunk_get_prev_ncnl(pc);
+   if ((prev == NULL) || !(prev->flags & PCF_IN_PREPROC))
+   {
+      return(false);
+   }
+
+   /* Find the chunk before the preprocessor */
+   while ((prev != NULL) && (prev->flags & PCF_IN_PREPROC))
+   {
+      prev = chunk_get_prev_ncnl(prev);
+   }
+
+   if ((prev != NULL) &&
+       (prev->parent_type == CT_DO) &&
+       ((prev->type == CT_VBRACE_CLOSE) ||
+        (prev->type == CT_BRACE_CLOSE)))
+   {
+      return(true);
+   }
+   return(false);
+}
 
 /**
  * At the heart of this algorithm are two stacks.
@@ -329,7 +357,8 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
       /* Make sure the open / close match */
       if (pc->type != (frm->pse[frm->pse_tos].type + 1))
       {
-         if (frm->pse[frm->pse_tos].type != CT_NONE)
+         if ((frm->pse[frm->pse_tos].type != CT_NONE) &&
+             (frm->pse[frm->pse_tos].type != CT_PP_DEFINE))
          {
             LOG_FMT(LWARN, "%s:%d Error: Unexpected '%.*s' for '%s'\n",
                     cpd.filename, pc->orig_line, pc->len, pc->str,
@@ -345,8 +374,9 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
          /* Copy the parent, update the paren/brace levels */
          pc->parent_type = frm->pse[frm->pse_tos].parent;
          frm->level--;
-         if ((pc->type == CT_BRACE_CLOSE) ||
-             (pc->type == CT_VBRACE_CLOSE))
+         if ((pc->type == CT_BRACE_CLOSE)  ||
+             (pc->type == CT_VBRACE_CLOSE) ||
+             (pc->type == CT_MACRO_CLOSE))
          {
             frm->brace_level--;
          }
@@ -365,12 +395,46 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
       }
    }
 
-   if ((pc->type == CT_SEMICOLON) &&
-       (frm->pse[frm->pse_tos].stage == BS_WOD_SEMI))
+   /* In this state, we expect a semicolon, but we'll also hit the closing
+    * sparen, so we need to check cpd.consumed to see if the close sparen was
+    * aleady handled.
+    */
+   if (frm->pse[frm->pse_tos].stage == BS_WOD_SEMI)
    {
-      cpd.consumed    = true;
-      pc->parent_type = CT_WHILE_OF_DO;
-      handle_complex_close(frm, pc);
+      chunk_t *tmp = pc;
+
+      if (cpd.consumed)
+      {
+         /* If consumed, then we are on the close sparen.
+          * PAWN: Check the next chunk for a semicolon. If it isn't, then
+          * add a virtual semicolon, which will get handled on the next pass.
+          */
+         if (cpd.lang_flags & LANG_PAWN)
+         {
+            tmp = chunk_get_next_ncnl(pc);
+
+            if ((tmp->type != CT_SEMICOLON) && (tmp->type != CT_VSEMICOLON))
+            {
+               pawn_add_vsemi_after(pc);
+            }
+         }
+      }
+      else
+      {
+         /* Complain if this ISN'T a semicolon, but close out WHILE_OF_DO anyway */
+         if ((pc->type == CT_SEMICOLON) || (pc->type == CT_VSEMICOLON))
+         {
+            cpd.consumed    = true;
+            pc->parent_type = CT_WHILE_OF_DO;
+         }
+         else
+         {
+            LOG_FMT(LWARN, "%s:%d: Error: Expected a semicolon for WHILE_OF_DO, but got '%s'\n",
+                    cpd.filename, pc->orig_line, get_token_name(pc->type));
+            cpd.error_count++;
+         }
+         handle_complex_close(frm, pc);
+      }
    }
 
    /* Get the parent type for brace and paren open */
@@ -468,8 +532,16 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    else if (patcls == PATCLS_PBRACED)
    {
       frm->pse_tos++;
-      frm->pse[frm->pse_tos].type  = pc->type;
-      frm->pse[frm->pse_tos].stage = BS_PAREN1;
+      if ((pc->type == CT_WHILE) && maybe_while_of_do(pc))
+      {
+         pc->type = CT_WHILE_OF_DO;
+         frm->pse[frm->pse_tos].stage = BS_WOD_PAREN;
+      }
+      else
+      {
+         frm->pse[frm->pse_tos].stage = BS_PAREN1;
+      }
+      frm->pse[frm->pse_tos].type = pc->type;
 
       print_stack(LBCSPUSH, "+ComplexParenBraced", frm, pc);
    }
@@ -481,6 +553,14 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
 
       print_stack(LBCSPUSH, "+ComplexOpParenBraced", frm, pc);
    }
+   else if (patcls == PATCLS_ELSE)
+   {
+      frm->pse_tos++;
+      frm->pse[frm->pse_tos].type  = pc->type;
+      frm->pse[frm->pse_tos].stage = BS_ELSEIF;
+
+      print_stack(LBCSPUSH, "+ComplexElse", frm, pc);
+   }
 
    /* Mark simple statement/expression starts
     *  - after { or }
@@ -530,7 +610,6 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
    }
 }
 
-
 /**
  * Checks the progression of complex statements.
  * - checks for else after if
@@ -591,13 +670,34 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
       frm->pse[frm->pse_tos].stage = BS_BRACE2;
    }
 
+   /* Check for CT_CATCH or CT_FINALLY after CT_TRY or CT_CATCH */
+   while (frm->pse[frm->pse_tos].stage == BS_CATCH)
+   {
+      if ((pc->type == CT_CATCH) || (pc->type == CT_FINALLY))
+      {
+         /* Replace CT_TRY with CT_CATCH on the stack & we are done */
+         frm->pse[frm->pse_tos].type  = pc->type;
+         frm->pse[frm->pse_tos].stage = (pc->type == CT_CATCH) ? BS_OP_PAREN1 : BS_BRACE2;
+         print_stack(LBCSSWAP, "=Swap   ", frm, pc);
+         return(true);
+      }
+
+      /* Remove the CT_TRY and close the statement */
+      frm->pse_tos--;
+      print_stack(LBCSPOP, "-TRY-CCS ", frm, pc);
+      if (close_statement(frm, pc))
+      {
+         return(true);
+      }
+   }
+
    /* Check for CT_WHILE after the CT_DO */
    if (frm->pse[frm->pse_tos].stage == BS_WHILE)
    {
       if (pc->type == CT_WHILE)
       {
          pc->type = CT_WHILE_OF_DO;
-         frm->pse[frm->pse_tos].type  = CT_WHILE_OF_DO;//CT_WHILE;
+         frm->pse[frm->pse_tos].type  = CT_WHILE_OF_DO; //CT_WHILE;
          frm->pse[frm->pse_tos].stage = BS_WOD_PAREN;
          return(true);
       }
@@ -660,7 +760,6 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
    return(false);
 }
 
-
 /**
  * Handles a close paren or brace - just progress the stage, if the end
  * of the statement is hit, call close_statement()
@@ -698,6 +797,25 @@ static bool handle_complex_close(struct parse_frame *frm, chunk_t *pc)
             }
          }
       }
+      else if ((frm->pse[frm->pse_tos].type == CT_TRY) ||
+               (frm->pse[frm->pse_tos].type == CT_CATCH))
+      {
+         frm->pse[frm->pse_tos].stage = BS_CATCH;
+
+         /* If the next chunk isn't CT_CATCH or CT_FINALLY, close the statement */
+         next = chunk_get_next_ncnl(pc);
+         if ((next != NULL) &&
+             (next->type != CT_CATCH) &&
+             (next->type != CT_FINALLY))
+         {
+            frm->pse_tos--;
+            print_stack(LBCSPOP, "-TRY-HCS ", frm, pc);
+            if (close_statement(frm, pc))
+            {
+               return(true);
+            }
+         }
+      }
       else
       {
          LOG_FMT(LNOTE, "%s: close_statement on %s BS_BRACE2\n", __func__,
@@ -745,7 +863,6 @@ static bool handle_complex_close(struct parse_frame *frm, chunk_t *pc)
    return(false);
 }
 
-
 static chunk_t *insert_vbrace(chunk_t *pc, bool after,
                               struct parse_frame *frm)
 {
@@ -790,7 +907,6 @@ static chunk_t *insert_vbrace(chunk_t *pc, bool after,
    return(rv);
 }
 
-
 /**
  * Called when a statement was just closed and the pse_tos was just
  * decremented.
