@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: chunk_list.cpp 611 2006-12-20 04:12:46Z bengardner $
+ * $Id: chunk_list.cpp 965 2007-10-15 00:18:47Z bengardner $
  */
 
 #include "chunk_list.h"
@@ -15,7 +15,7 @@
 #include "ListManager.h"
 #include "prototypes.h"
 
-typedef ListManager <chunk_t>   ChunkList;
+typedef ListManager<chunk_t>   ChunkList;
 
 ChunkList g_cl;
 
@@ -74,24 +74,46 @@ chunk_t *chunk_add(const chunk_t *pc_in)
    return(pc);
 }
 
+/**
+ * Add a copy after the given chunk.
+ * If ref is NULL, add at the head.
+ */
 chunk_t *chunk_add_after(const chunk_t *pc_in, chunk_t *ref)
 {
    chunk_t *pc;
 
    if ((pc = chunk_dup(pc_in)) != NULL)
    {
-      g_cl.AddAfter(pc, ref);
+      if (ref != NULL)
+      {
+         g_cl.AddAfter(pc, ref);
+      }
+      else
+      {
+         g_cl.AddHead(pc);
+      }
    }
    return(pc);
 }
 
+/**
+ * Add a copy before the given chunk.
+ * If ref is NULL, add at the head.
+ */
 chunk_t *chunk_add_before(const chunk_t *pc_in, chunk_t *ref)
 {
    chunk_t *pc;
 
    if ((pc = chunk_dup(pc_in)) != NULL)
    {
-      g_cl.AddBefore(pc, ref);
+      if (ref != NULL)
+      {
+         g_cl.AddBefore(pc, ref);
+      }
+      else
+      {
+         g_cl.AddTail(pc);
+      }
    }
    return(pc);
 }
@@ -411,7 +433,6 @@ bool chunk_is_newline_between(chunk_t *start, chunk_t *end)
    return(false);
 }
 
-
 /**
  * Swaps the two chunks.
  *
