@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: defines.cpp 689 2007-02-28 00:53:22Z bengardner $
+ * $Id: defines.cpp 971 2007-10-17 21:26:22Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -39,7 +39,6 @@ static int def_compare(const void *p1, const void *p2)
    return(strcmp(t1->tag, t2->tag));
 }
 
-
 /**
  * Adds an entry to the define list
  *
@@ -101,7 +100,6 @@ void add_define(const char *tag, const char *value)
    }
 }
 
-
 /**
  * Search the define table for a match
  *
@@ -118,7 +116,7 @@ const define_tag_t *find_define(const char *word, int len)
    if (len > (int)(sizeof(buf) - 1))
    {
       LOG_FMT(LNOTE, "%s: define too long at %d char (%d max) : %.*s\n",
-              __func__, len, sizeof(buf), len, word);
+              __func__, len, (int)sizeof(buf), len, word);
       return(NULL);
    }
    memcpy(buf, word, len);
@@ -132,7 +130,6 @@ const define_tag_t *find_define(const char *word, int len)
    return(p_ret);
 }
 
-
 /**
  * Loads the defines from a file
  *
@@ -172,7 +169,7 @@ int load_define_file(const char *filename)
 
       if (argc > 0)
       {
-         if ((argc <= 2) && ((get_char_table(*args[0]) & CT_KW1) != 0))
+         if ((argc <= 2) && CharTable::IsKw1(*args[0]))
          {
             LOG_FMT(LDEFVAL, "%s: line %d - %s\n", filename, line_no, args[0]);
             add_define(args[0], args[1]);
@@ -211,7 +208,7 @@ void output_defines(FILE *pfile)
    }
 }
 
-const define_tag_t *get_define_idx(int &idx)
+const define_tag_t *get_define_idx(int& idx)
 {
    const define_tag_t *dt = NULL;
 
