@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: indent.cpp 708 2007-03-06 02:16:16Z bengardner $
+ * $Id: indent.cpp 997 2007-11-08 00:06:09Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -136,14 +136,14 @@ void indent_to_column(chunk_t *pc, int column)
  * @param pc      The chunk at the start of the line
  * @param column  The desired column
  */
-void reindent_line(chunk_t *pc, int column)
+void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
 {
    int col_delta;
    int min_col;
 
-   LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d\n",
+   LOG_FMT(LINDLINE, "%s: %d] col %d on %.*s [%s] => %d <called from '%s' line %d\n",
            __func__, pc->orig_line, pc->column, pc->len, pc->str,
-           get_token_name(pc->type), column);
+           get_token_name(pc->type), column, fcn_name, lineno);
 
    if (column == pc->column)
    {
@@ -156,7 +156,11 @@ void reindent_line(chunk_t *pc, int column)
    do
    {
       min_col += pc->len;
-      pc       = chunk_get_next(pc);
+      if (pc->flags & PCF_FORCE_SPACE)
+      {
+         min_col++;
+      }
+      pc = chunk_get_next(pc);
       if (pc != NULL)
       {
          if (chunk_is_comment(pc) && (pc->parent_type != CT_COMMENT_EMBED))
@@ -181,7 +185,6 @@ void reindent_line(chunk_t *pc, int column)
    } while ((pc != NULL) && (pc->nl_count == 0));
 }
 
-
 /**
  * Starts a new entry
  *
@@ -199,9 +202,10 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
       frm.pse_tos++;
       memset(&frm.pse[frm.pse_tos], 0, sizeof(frm.pse[frm.pse_tos]));
 
-      LOG_FMT(LINDPSE, "%4d] OPEN  [%d,%s] level=%d\n",
-              pc->orig_line, frm.pse_tos, get_token_name(pc->type), pc->level);
+      LOG_FMT(LINDPSE, "%4d] (pp=%d) OPEN  [%d,%s] level=%d\n",
+              pc->orig_line, cpd.pp_level, frm.pse_tos, get_token_name(pc->type), pc->level);
 
+      frm.pse[frm.pse_tos].pc         = pc;
       frm.pse[frm.pse_tos].type       = pc->type;
       frm.pse[frm.pse_tos].level      = pc->level;
       frm.pse[frm.pse_tos].open_line  = pc->orig_line;
@@ -211,7 +215,6 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
    }
 }
 
-
 /**
  * Removes the top entry
  *
@@ -225,8 +228,8 @@ static void indent_pse_pop(struct parse_frame& frm, chunk_t *pc)
    {
       if (pc != NULL)
       {
-         LOG_FMT(LINDPSE, "%4d] CLOSE [%d,%s] on %s, started on line %d, level=%d/%d\n",
-                 pc->orig_line, frm.pse_tos,
+         LOG_FMT(LINDPSE, "%4d] (pp=%d) CLOSE [%d,%s] on %s, started on line %d, level=%d/%d\n",
+                 pc->orig_line, cpd.pp_level, frm.pse_tos,
                  get_token_name(frm.pse[frm.pse_tos].type),
                  get_token_name(pc->type),
                  frm.pse[frm.pse_tos].open_line,
@@ -239,11 +242,14 @@ static void indent_pse_pop(struct parse_frame& frm, chunk_t *pc)
                  frm.pse_tos, get_token_name(frm.pse[frm.pse_tos].type),
                  frm.pse[frm.pse_tos].open_line);
       }
+
+      /* Don't clear the stack entry because some code 'cheats' and uses the
+       * just-popped indent values
+       */
       frm.pse_tos--;
    }
 }
 
-
 static int token_indent(c_token_t type)
 {
    switch (type)
@@ -270,7 +276,6 @@ static int token_indent(c_token_t type)
    }
 }
 
-
 /**
  * Change the top-level indentation only by changing the column member in
  * the chunk structures.
@@ -294,12 +299,15 @@ void indent_text(void)
    int                parent_token_indent = 0;
    int                xml_indent          = 0;
    bool               token_used;
+   int                sql_col             = 0;
+   int                sql_orig_col        = 0;
 
    memset(&frm, 0, sizeof(frm));
 
    /* dummy top-level entry */
    frm.pse[0].indent     = 1;
    frm.pse[0].indent_tmp = 1;
+   frm.pse[0].indent_tab = 1;
    frm.pse[0].type       = CT_EOF;
 
    pc = chunk_get_head();
@@ -314,27 +322,131 @@ void indent_text(void)
          parent_token_indent = token_indent(pc->parent_type);
       }
 
-      /* Clean up after a #define */
+      /* Clean up after a #define, etc */
       if (!in_preproc)
       {
          while ((frm.pse_tos > 0) && frm.pse[frm.pse_tos].in_preproc)
          {
+            c_token_t type = frm.pse[frm.pse_tos].type;
             indent_pse_pop(frm, pc);
+
+            /* If we just removed an #endregion, then check to see if a
+             * PP_REGION_INDENT entry is right below it
+             */
+            if ((type == CT_PP_ENDREGION) &&
+                (frm.pse[frm.pse_tos].type == CT_PP_REGION_INDENT))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
+            /* If we just removed an #endregion, then check to see if a
+             * PP_REGION_INDENT entry is right below it
+             */
+            if (((type == CT_PP_ENDIF) || (type == CT_PP_ELSE)) &&
+                (frm.pse[frm.pse_tos].type == CT_PP_IF_INDENT))
+            {
+               indent_pse_pop(frm, pc);
+            }
          }
       }
-      else
+      else if (pc->type == CT_PREPROC)
       {
+         /* Close out PP_IF_INDENT before playing with the parse frames */
+         if ((frm.pse[frm.pse_tos].type == CT_PP_IF_INDENT) &&
+             ((pc->parent_type == CT_PP_ENDIF) ||
+              (pc->parent_type == CT_PP_ELSE)))
+         {
+            indent_pse_pop(frm, pc);
+         }
+
          pf_check(&frm, pc);
 
-         if (!was_preproc)
+         /* Indent the body of a #region here */
+         if (cpd.settings[UO_pp_region_indent_code].b &&
+             (pc->parent_type == CT_PP_REGION))
          {
-            /* Transition into a preproc by creating a dummy indent */
-            frm.level++;
-            indent_pse_push(frm, pc);
+            next = chunk_get_next(pc);
+            /* Hack to get the logs to look right */
+            next->type = CT_PP_REGION_INDENT;
+            indent_pse_push(frm, next);
+            next->type = CT_PP_REGION;
 
-            frm.pse[frm.pse_tos].indent     = 1 + indent_size;
+            /* Indent one level */
+            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;;
+            frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
+            frm.pse[frm.pse_tos].in_preproc = false;
+         }
+
+         /* Indent the body of a #if here */
+         if (cpd.settings[UO_pp_if_indent_code].b &&
+             ((pc->parent_type == CT_PP_IF) ||
+              (pc->parent_type == CT_PP_ELSE)))
+         {
+            next = chunk_get_next(pc);
+            /* Hack to get the logs to look right */
+            next->type = CT_PP_IF_INDENT;
+            indent_pse_push(frm, next);
+            next->type = CT_PP_IF;
+
+            /* Indent one level */
+            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;;
+            frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
+            frm.pse[frm.pse_tos].in_preproc = false;
+         }
+
+         /* Transition into a preproc by creating a dummy indent */
+         frm.level++;
+         indent_pse_push(frm, chunk_get_next(pc));
+
+         if (pc->parent_type == CT_PP_DEFINE)
+         {
+            frm.pse[frm.pse_tos].indent_tmp = cpd.settings[UO_pp_define_at_level].b ?
+                                              frm.pse[frm.pse_tos - 1].indent_tmp : 1;
+            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos].indent_tmp + indent_size;
+         }
+         else
+         {
+            if ((frm.pse[frm.pse_tos - 1].type == CT_PP_REGION_INDENT) ||
+                (frm.pse[frm.pse_tos - 1].type == CT_PP_IF_INDENT))
+            {
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 2].indent;
+            }
+            else
+            {
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent;
+            }
+            if ((pc->parent_type == CT_PP_REGION) ||
+                (pc->parent_type == CT_PP_ENDREGION))
+            {
+               int val = cpd.settings[UO_pp_indent_region].n;
+               if (val > 0)
+               {
+                  frm.pse[frm.pse_tos].indent = val;
+               }
+               else
+               {
+                  frm.pse[frm.pse_tos].indent += val;
+               }
+            }
+            else if ((pc->parent_type == CT_PP_IF) ||
+                     (pc->parent_type == CT_PP_ELSE) ||
+                     (pc->parent_type == CT_PP_ENDIF))
+            {
+               int val = cpd.settings[UO_pp_indent_if].n;
+               if (val > 0)
+               {
+                  frm.pse[frm.pse_tos].indent = val;
+               }
+               else
+               {
+                  frm.pse[frm.pse_tos].indent += val;
+               }
+            }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          }
+
       }
 
       if ((cout_col > 0) &&
@@ -403,7 +515,14 @@ void indent_text(void)
             if ((frm.pse[frm.pse_tos].type == CT_ASSIGN) &&
                 (chunk_is_semicolon(pc) ||
                  (pc->type == CT_COMMA) ||
-                 (pc->type == CT_BRACE_OPEN)))
+                 (pc->type == CT_BRACE_OPEN) ||
+                 ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
+            /* End any assign operations with a semicolon on the same level */
+            if ((frm.pse[frm.pse_tos].type == CT_IMPORT) && chunk_is_semicolon(pc))
             {
                indent_pse_pop(frm, pc);
             }
@@ -660,7 +779,7 @@ void indent_text(void)
          indent_pse_push(frm, pc);
 
          frm.pse[frm.pse_tos].indent     = tmp;
-         frm.pse[frm.pse_tos].indent_tmp = tmp - indent_size;
+         frm.pse[frm.pse_tos].indent_tmp = tmp - indent_size + cpd.settings[UO_indent_case_shift].n;
          frm.pse[frm.pse_tos].indent_tab = tmp;
 
          /* Always set on case statements */
@@ -710,6 +829,14 @@ void indent_text(void)
                frm.pse[frm.pse_tos].indent += 2;
                /* don't change indent of current line */
             }
+            else
+            {
+               next = chunk_get_next(pc);
+               if ((next != NULL) && !chunk_is_newline(next))
+               {
+                  frm.pse[frm.pse_tos].indent = next->column;
+               }
+            }
          }
       }
       else if ((pc->type == CT_PAREN_OPEN) ||
@@ -729,8 +856,8 @@ void indent_text(void)
             frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
          }
 
-         if ((chunk_is_str(pc, "(", 1) && !cpd.settings[UO_indent_paren_nl].b) ||
-             (chunk_is_str(pc, "[", 1) && !cpd.settings[UO_indent_square_nl].b))
+         else if ((chunk_is_str(pc, "(", 1) && !cpd.settings[UO_indent_paren_nl].b) ||
+                  (chunk_is_str(pc, "[", 1) && !cpd.settings[UO_indent_square_nl].b))
          {
             next = chunk_get_next_nc(pc);
             if (chunk_is_newline(next))
@@ -742,11 +869,19 @@ void indent_text(void)
                }
                frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - sub].indent + indent_size;
             }
+            else
+            {
+               if (!chunk_is_comment(next))
+               {
+                  frm.pse[frm.pse_tos].indent = next->column;
+               }
+            }
          }
+
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.paren_count++;
       }
-      else if (pc->type == CT_ASSIGN)
+      else if ((pc->type == CT_ASSIGN) || (pc->type == CT_IMPORT))
       {
          /**
           * if there is a newline after the '=', just indent one level,
@@ -810,11 +945,22 @@ void indent_text(void)
           */
 
          prev = chunk_get_prev_ncnl(pc);
-         if ((pc->type == CT_MEMBER) ||
-             (pc->type == CT_DC_MEMBER) ||
-             ((prev != NULL) &&
-              ((prev->type == CT_MEMBER) ||
-               (prev->type == CT_DC_MEMBER))))
+         if ((pc->flags & PCF_DONT_INDENT) != 0)
+         {
+            /* no change */
+         }
+         else if ((pc->parent_type == CT_SQL_EXEC) &&
+                  cpd.settings[UO_indent_preserve_sql].b)
+         {
+            reindent_line(pc, sql_col + (pc->orig_col - sql_orig_col));
+            LOG_FMT(LSYS, "Indent SQL: [%.*s] to %d (%d/%d)\n",
+                    pc->len, pc->str, pc->column, sql_col, sql_orig_col);
+         }
+         else if ((pc->type == CT_MEMBER) ||
+                  (pc->type == CT_DC_MEMBER) ||
+                  ((prev != NULL) &&
+                   ((prev->type == CT_MEMBER) ||
+                    (prev->type == CT_DC_MEMBER))))
          {
             tmp = cpd.settings[UO_indent_member].n + indent_column;
             LOG_FMT(LINDENT, "%s: %d] member => %d\n",
@@ -853,11 +999,63 @@ void indent_text(void)
          }
          else if (pc->type == CT_PREPROC)
          {
-            /* Preprocs are always in column 1. See indent_preproc() */
-            if (pc->column != 1)
+            LOG_FMT(LINDENT, "%s: %d] pp-indent => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, indent_column);
+         }
+         else if (chunk_is_paren_close(pc))
+         {
+            /* This is a big hack. We assume that since we hit a paren close,
+             * that we just removed a paren open */
+            if (frm.pse[frm.pse_tos + 1].type == c_token_t(pc->type - 1))
             {
-               reindent_line(pc, 1);
+               chunk_t *ck1 = frm.pse[frm.pse_tos + 1].pc;
+               chunk_t *ck2 = chunk_get_prev(ck1);
+
+               /* If the open paren was the first thing on the line or we are
+                * doing mode 1, then put the close paren in the same column */
+               if (chunk_is_newline(ck2) ||
+                   (cpd.settings[UO_indent_paren_close].n == 1))
+               {
+                  indent_column = ck1->column;
+               }
+               else
+               {
+                  if (cpd.settings[UO_indent_paren_close].n != 2)
+                  {
+                     indent_column = frm.pse[frm.pse_tos + 1].indent_tmp;
+                     if (cpd.settings[UO_indent_paren_close].n == 1)
+                     {
+                        indent_column--;
+                     }
+                  }
+               }
             }
+            LOG_FMT(LINDENT, "%s: %d] cl paren => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, indent_column);
+         }
+         else if (pc->type == CT_COMMA)
+         {
+            if (cpd.settings[UO_indent_comma_paren].b &&
+                chunk_is_paren_open(frm.pse[frm.pse_tos].pc))
+            {
+               indent_column = frm.pse[frm.pse_tos].pc->column;
+            }
+            LOG_FMT(LINDENT, "%s: %d] comma => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, indent_column);
+         }
+         else if (pc->type == CT_BOOL)
+         {
+            if (cpd.settings[UO_indent_bool_paren].b &&
+                chunk_is_paren_open(frm.pse[frm.pse_tos].pc))
+            {
+               indent_column = frm.pse[frm.pse_tos].pc->column;
+            }
+            LOG_FMT(LINDENT, "%s: %d] bool => %d [%.*s]\n",
+                    __func__, pc->orig_line, indent_column, pc->len, pc->str);
+            reindent_line(pc, indent_column);
          }
          else
          {
@@ -869,6 +1067,14 @@ void indent_text(void)
             }
          }
          did_newline = false;
+
+         if ((pc->type == CT_SQL_EXEC) ||
+             (pc->type == CT_SQL_BEGIN) ||
+             (pc->type == CT_SQL_END))
+         {
+            sql_col      = pc->column;
+            sql_orig_col = pc->orig_col;
+         }
       }
 
       /**
@@ -948,7 +1154,6 @@ void indent_text(void)
    }
 }
 
-
 /**
  * returns true if forward scan reveals only single newlines or comments
  * stops when hits code
@@ -1083,25 +1288,14 @@ static void indent_comment(chunk_t *pc, int col)
    reindent_line(pc, col);
 }
 
-
 /**
- * Put spaces on either side of the preproc (#) symbol.
- * This is done by pointing pc->str into pp_str and adjusting the
- * length.
+ * Scan to see if the whole file is covered by one #ifdef
  */
-void indent_preproc(void)
+static bool ifdef_over_whole_file()
 {
    chunk_t *pc;
    chunk_t *next;
-   int     pp_level;
-   int     pp_level_sub = 0;
-   int     tmp;
 
-   /* Define a string of 16 spaces + # + 16 spaces */
-   static const char *pp_str  = "                #                ";
-   static const char *alt_str = "                %:                ";
-
-   /* Scan to see if the whole file is covered by one #ifdef */
    int stage = 0;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
@@ -1150,6 +1344,26 @@ void indent_preproc(void)
    if (stage == 2)
    {
       LOG_FMT(LINFO, "The whole file is covered by a #IF\n");
+      return(true);
+   }
+   return(stage == 2);
+}
+
+/**
+ * Indent the preprocessor stuff from column 1.
+ * FIXME: This is broken if there is a comment or escaped newline
+ * between '#' and 'define'.
+ */
+void indent_preproc(void)
+{
+   chunk_t *pc;
+   chunk_t *next;
+   int     pp_level;
+   int     pp_level_sub = 0;
+
+   /* Scan to see if the whole file is covered by one #ifdef */
+   if (ifdef_over_whole_file())
+   {
       pp_level_sub = 1;
    }
 
@@ -1160,62 +1374,57 @@ void indent_preproc(void)
          continue;
       }
 
-      if (pc->column != 1)
-      {
-         /* Don't handle preprocessors that aren't in column 1 */
-         LOG_FMT(LINFO, "%s: Line %d doesn't start in column 1 (%d)\n",
-                 __func__, pc->orig_line, pc->column);
-         continue;
-      }
-
-      /* point into pp_str */
-      if (pc->len == 2)
-      {
-         /* alternate token crap */
-         pc->str = &alt_str[16];
-      }
-      else
-      {
-         pc->str = &pp_str[16];
-      }
+      next = chunk_get_next_ncnl(pc);
 
       pp_level = pc->pp_level - pp_level_sub;
       if (pp_level < 0)
       {
          pp_level = 0;
       }
-      else if (pp_level > 16)
-      {
-         pp_level = 16;
-      }
 
-      /* Note that the indent is removed by default */
+      /* Adjust the indent of the '#' */
       if ((cpd.settings[UO_pp_indent].a & AV_ADD) != 0)
       {
-         /* Need to add some spaces */
-         pc->str -= pp_level;
-         pc->len += pp_level;
+         reindent_line(pc, 1 + pp_level);
       }
-      else if (cpd.settings[UO_pp_indent].a == AV_IGNORE)
+      else if ((cpd.settings[UO_pp_indent].a & AV_REMOVE) != 0)
       {
-         tmp      = (pc->orig_col <= 16) ? pc->orig_col - 1 : 16;
-         pc->str -= tmp;
-         pc->len += tmp;
+         reindent_line(pc, 1);
       }
 
       /* Add spacing by adjusting the length */
-      if ((cpd.settings[UO_pp_space].a & AV_ADD) != 0)
+      if ((cpd.settings[UO_pp_space].a != AV_IGNORE) && (next != NULL))
       {
-         pc->len += pp_level;
+         if ((cpd.settings[UO_pp_space].a & AV_ADD) != 0)
+         {
+            int mult = cpd.settings[UO_pp_space_count].n;
+
+            if (mult < 1)
+            {
+               mult = 1;
+            }
+            reindent_line(next, pc->column + pc->len + (pp_level * mult));
+         }
+         else if ((cpd.settings[UO_pp_space].a & AV_REMOVE) != 0)
+         {
+            reindent_line(next, pc->column + pc->len);
+         }
       }
 
-      next = chunk_get_next(pc);
-      if (next != NULL)
+      /* Mark as already handled if not region stuff or in column 1 */
+      if ((!cpd.settings[UO_pp_indent_at_level].b ||
+           (pc->brace_level <= (pc->parent_type == CT_PP_DEFINE ? 1 : 0))) &&
+          (pc->parent_type != CT_PP_REGION) &&
+          (pc->parent_type != CT_PP_ENDREGION))
       {
-         reindent_line(next, pc->len + 1);
+         if (!cpd.settings[UO_pp_define_at_level].b ||
+             (pc->parent_type != CT_PP_DEFINE))
+         {
+            pc->flags |= PCF_DONT_INDENT;
+         }
       }
 
       LOG_FMT(LPPIS, "%s: Indent line %d to %d (len %d, next->col %d)\n",
-              __func__, pc->orig_line, pp_level, pc->len, next->column);
+              __func__, pc->orig_line, 1 + pp_level, pc->len, next->column);
    }
 }
