@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: log_levels.h 667 2007-02-17 03:31:53Z bengardner $
+ * $Id: log_levels.h 987 2007-11-02 18:53:04Z bengardner $
  */
 #ifndef LOG_LEVELS_H_INCLUDED
 #define LOG_LEVELS_H_INCLUDED
@@ -68,7 +68,13 @@ enum
    LTEMPL    = 52,    /* Template detection */
    LINDENT2  = 53,    /* indent_text tab level */
    LPARADD   = 54,    /* adding parens in if/while */
-   LCMTNL    = 55,    /* newlines before comments */
+   LPARADD2  = 55,    /* adding parens in if/while - details */
+   LCMTNL    = 56,    /* newlines before comments */
+   LTEMPFUNC = 57,    /* Template function detection */
+   LDELSEMI  = 58,    /* Removing semicolons */
+   LFPARAM   = 59,    /* Testing for a full parameter */
+   LNL1LINE  = 60,    /* NL check for 1 liners */
+   LPFCHK    = 61,    /* Parse Frame check fcn call */
 };
 
 #endif   /* LOG_LEVELS_H_INCLUDED */
