@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: newlines.cpp 697 2007-02-28 02:03:30Z bengardner $
+ * $Id: newlines.cpp 991 2007-11-05 14:53:21Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -16,6 +16,11 @@
 #include <cerrno>
 #include <cctype>
 
+
+static void newlines_double_space_struct_enum_union(chunk_t *open_brace);
+static bool one_liner_nl_ok(chunk_t *pc);
+
+
 /*
  * Basic approach:
  * 1. Find next open brace
@@ -133,12 +138,26 @@ static void newline_min_after2(chunk_t *ref, INT32 count,
    }
 }
 
-
 /**
  * Adds a newline between the two chunks if there isn't one already.
  */
 #define newline_add_between(start, end)     newline_add_between2(start, end, __func__, __LINE__)
 
+/**
+ * Add a newline between two tokens.
+ * If there is already a newline between then, nothing is done.
+ * Otherwise a newline is inserted.
+ *
+ * If end is CT_BRACE_OPEN and a comment and newline follow, then
+ * the brace open is moved instead of inserting a newline.
+ *
+ * In this situation:
+ *    if (...) { //comment
+ *
+ * you get:
+ *    if (...)   //comment
+ *    {
+ */
 static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
                                      const char *func, int line)
 {
@@ -149,14 +168,13 @@ static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
       return(NULL);
    }
 
-   LOG_FMT(LNEWLINE, "%s: '%.*s' line %d:%d and '%.*s' line %d:%d : caller=%s:%d\n",
-           __func__, start->len, start->str, start->orig_line, start->orig_col,
+   LOG_FMT(LNEWLINE, "%s: '%.*s'[%s] line %d:%d and '%.*s' line %d:%d : caller=%s:%d\n",
+           __func__, start->len, start->str, get_token_name(start->type),
+           start->orig_line, start->orig_col,
            end->len, end->str, end->orig_line, end->orig_col, func, line);
 
-   if (((start->type == CT_BRACE_OPEN) &&
-        ((start->flags & PCF_ONE_CLASS) == PCF_ONE_CLASS)) ||
-       ((end->type == CT_BRACE_CLOSE) &&
-        ((end->flags & PCF_ONE_CLASS) == PCF_ONE_CLASS)))
+   /* Back-up check for one-liners (should never be true!) */
+   if (!one_liner_nl_ok(start))
    {
       return(NULL);
    }
@@ -164,19 +182,38 @@ static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
    /* Scan for a line break */
    for (pc = start; pc != end; pc = chunk_get_next(pc))
    {
-      if ((pc->type == CT_NEWLINE) || (pc->type == CT_NL_CONT))
+      if (chunk_is_newline(pc))
       {
          return(pc);
       }
    }
 
+   /* If the second one is a brace open, then check to see
+    * if a comment + newline follows
+    */
+   if (end->type == CT_BRACE_OPEN)
+   {
+      pc = chunk_get_next(end);
+      if (chunk_is_comment(pc))
+      {
+         pc = chunk_get_next(pc);
+         if (chunk_is_newline(pc))
+         {
+            /* Move the open brace to after the newline */
+            chunk_move_after(end, pc);
+            return(pc);
+         }
+      }
+   }
+
    return(newline_add_before(end));
 }
 
-
 /**
  * Removes any CT_NEWLINE or CT_NL_CONT between start and end.
  * Start must be before end on the chunk list.
+ * If the 'PCF_IN_PREPROC' status differs between two tags, we can't remove
+ * the newline.
  *
  * @param start   The starting chunk (cannot be a newline)
  * @param end     The ending chunk (cannot be a newline)
@@ -193,9 +230,17 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
    chunk_t *pc    = start;
    bool    retval = false;
 
-   LOG_FMT(LNEWLINE, "%s: '%.*s' line %d:%d and '%.*s' line %d:%d : caller=%s:%d\n",
+   LOG_FMT(LNEWLINE, "%s: '%.*s' line %d:%d and '%.*s' line %d:%d : caller=%s:%d preproc=%d/%d\n",
            __func__, start->len, start->str, start->orig_line, start->orig_col,
-           end->len, end->str, end->orig_line, end->orig_col, func, line);
+           end->len, end->str, end->orig_line, end->orig_col, func, line,
+           ((start->flags & PCF_IN_PREPROC) != 0),
+           ((end->flags & PCF_IN_PREPROC) != 0));
+
+   /* Can't remove anything if the preproc status differs */
+   if (!chunk_same_preproc(start, end))
+   {
+      return(false);
+   }
 
    do
    {
@@ -203,15 +248,14 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
       if (chunk_is_newline(pc))
       {
          prev = chunk_get_prev(pc);
-         if (!chunk_is_comment(prev) && !chunk_is_comment(next))
+         if ((!chunk_is_comment(prev) && !chunk_is_comment(next)) ||
+             chunk_is_newline(prev) ||
+             chunk_is_newline(next))
          {
-            chunk_del(pc);
-            retval = true;
-         }
-         else if (chunk_is_newline(prev) ||
-                  chunk_is_newline(next))
-         {
-            chunk_del(pc);
+            if (chunk_safe_to_del_nl(pc))
+            {
+               chunk_del(pc);
+            }
             retval = true;
          }
          else
@@ -239,7 +283,6 @@ static bool newline_del_between2(chunk_t *start, chunk_t *end,
    return(retval);
 }
 
-
 /**
  * Add or remove a newline between the closing paren and opening brace.
  * Also uncuddles anything on the closing brace. (may get fixed later)
@@ -265,8 +308,22 @@ static void newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
       close_paren = chunk_get_next_type(pc, CT_SPAREN_CLOSE, pc->level);
       brace_open  = chunk_get_next_ncnl(close_paren);
 
-      if ((brace_open != NULL) && (brace_open->type == CT_BRACE_OPEN))
+      if ((brace_open != NULL) && (brace_open->type == CT_BRACE_OPEN) &&
+          one_liner_nl_ok(brace_open))
       {
+         if (cpd.settings[UO_nl_multi_line_cond].b)
+         {
+            nl_opt = AV_REMOVE;
+            while ((pc = chunk_get_next(pc)) != close_paren)
+            {
+               if (chunk_is_newline(pc))
+               {
+                  nl_opt = AV_ADD;
+                  break;
+               }
+            }
+         }
+
          if (nl_opt & AV_ADD)
          {
             newline_add_between(close_paren, brace_open);
@@ -318,7 +375,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
       {
          last_nl = pc;
          /* if we found 2 or more in a row */
-         if (pc->nl_count > 1 || chunk_is_newline(chunk_get_prev_nvb(pc)))
+         if ((pc->nl_count > 1) || chunk_is_newline(chunk_get_prev_nvb(pc)))
          {
             /* need to remove */
             if (nl_opt & AV_REMOVE)
@@ -328,7 +385,11 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
                /* can keep using pc because anything other than newline stops loop, and we delete if newline */
                while (chunk_is_newline(prev = chunk_get_prev_nvb(pc)))
                {
-                  chunk_del(prev);
+                  /* Make sure we don't combine a preproc and non-preproc */
+                  if (chunk_safe_to_del_nl(prev))
+                  {
+                     chunk_del(prev);
+                  }
                }
             }
 
@@ -376,12 +437,12 @@ static chunk_t *get_closing_brace(chunk_t *start)
 
    for (pc = start; (pc = chunk_get_next(pc)) != NULL;)
    {
-      if ((pc->type == CT_BRACE_CLOSE || pc->type == CT_VBRACE_CLOSE) && pc->level == level)
+      if (((pc->type == CT_BRACE_CLOSE) || (pc->type == CT_VBRACE_CLOSE)) && (pc->level == level))
       {
          return(pc);
       }
       /* for some reason, we can have newlines between if and opening brace that are lower level than either */
-      if (!chunk_is_newline(pc) && pc->level < level)
+      if (!chunk_is_newline(pc) && (pc->level < level))
       {
          return(NULL);
       }
@@ -402,7 +463,10 @@ static void remove_next_newlines(chunk_t *start)
    {
       if (chunk_is_newline(next))
       {
-         chunk_del(next);
+         if (chunk_safe_to_del_nl(next))
+         {
+            chunk_del(next);
+         }
       }
       else if (chunk_is_vbrace(next))
       {
@@ -449,7 +513,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
       while (true)
       {
          next = chunk_get_next_ncnl(pc);
-         if (next != NULL && (next->type == CT_ELSE || next->type == CT_ELSEIF))
+         if ((next != NULL) && ((next->type == CT_ELSE) || (next->type == CT_ELSEIF)))
          {
             /* point to the closing brace of the else */
             if ((pc = get_closing_brace(next)) == NULL)
@@ -542,7 +606,6 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
    }
 }
 
-
 /**
  * Adds or removes a newline between the keyword and the open brace.
  * If there is something after the '{' on the same line, then
@@ -602,7 +665,6 @@ static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt)
    }
 }
 
-
 /**
  * Cuddles or un-cuddles a chunk with a previous close brace
  *
@@ -636,7 +698,6 @@ static void newlines_cuddle_uncuddle(chunk_t *start, argval_t nl_opt)
    }
 }
 
-
 /**
  * Adds/removes a newline between else and '{'.
  * "else {" or "else \n {"
@@ -655,9 +716,18 @@ static void newlines_do_else(chunk_t *start, argval_t nl_opt)
    next = chunk_get_next_ncnl(start);
    if ((next != NULL) && (next->type == CT_BRACE_OPEN))
    {
+      if (!one_liner_nl_ok(next))
+      {
+         return;
+      }
       if ((nl_opt & AV_ADD) != 0)
       {
          newline_add_between(start, next);
+         chunk_t *tmp = chunk_get_next(next);
+         if ((tmp != NULL) && !chunk_is_newline(tmp))
+         {
+            newline_add_between(next, tmp);
+         }
       }
       else if ((nl_opt & AV_REMOVE) != 0)
       {
@@ -708,7 +778,6 @@ static void newline_fnc_var_def(chunk_t *br_open, int nl_count)
    }
 }
 
-
 /**
  * Handles the brace_on_func_line setting and decides if the closing brace
  * of a pair should be right after a newline.
@@ -757,7 +826,10 @@ static void newlines_brace_pair(chunk_t *br_open)
             next = chunk_get_next(pc);
             if (pc->type == CT_NEWLINE)
             {
-               chunk_del(pc);
+               if (chunk_safe_to_del_nl(pc))
+               {
+                  chunk_del(pc);
+               }
             }
             pc = next;
          }
@@ -765,7 +837,8 @@ static void newlines_brace_pair(chunk_t *br_open)
       }
    }
 
-   if ((br_open->flags & PCF_ONE_CLASS) == PCF_ONE_CLASS)
+   /* Make sure we don't break a one-liner */
+   if (!one_liner_nl_ok(br_open))
    {
       return;
    }
@@ -878,18 +951,6 @@ static void newlines_brace_pair(chunk_t *br_open)
    {
       newline_del_between(prev, br_close);
    }
-
-   if (cpd.settings[UO_eat_blanks_before_close_brace].b)
-   {
-      pc = chunk_get_prev_nc(br_close);
-      if (chunk_is_newline(pc))
-      {
-         if (pc->nl_count > 1)
-         {
-            pc->nl_count = 1;
-         }
-      }
-   }
 }
 
 /**
@@ -960,7 +1021,6 @@ static void newline_case_colon(chunk_t *start)
    }
 }
 
-
 /**
  * Put a empty line after a return statement, unless it is followed by a
  * close brace.
@@ -997,7 +1057,6 @@ static void newline_return(chunk_t *start)
    }
 }
 
-
 /**
  * Does a simple Ignore, Add, Remove, or Force after the given chunk
  *
@@ -1027,7 +1086,6 @@ static void newline_iarf(chunk_t *pc, argval_t av)
    }
 }
 
-
 /**
  * Formats a function declaration
  * Start points to the open paren
@@ -1038,10 +1096,13 @@ static void newline_func_def(chunk_t *start)
    chunk_t *prev = NULL;
    chunk_t *tmp;
 
-   pc = chunk_get_next_ncnl(start);
-   if (chunk_is_str(pc, ")", 1))
+   if (cpd.settings[UO_nl_func_paren].a != AV_IGNORE)
    {
-      return;
+      prev = chunk_get_prev_ncnl(start);
+      if (prev != NULL)
+      {
+         newline_iarf(prev, cpd.settings[UO_nl_func_paren].a);
+      }
    }
 
    /* Handle break newlines type and function */
@@ -1065,6 +1126,12 @@ static void newline_func_def(chunk_t *start)
       }
    }
 
+   pc = chunk_get_next_ncnl(start);
+   if (chunk_is_str(pc, ")", 1))
+   {
+      return;
+   }
+
    newline_iarf(start, cpd.settings[UO_nl_func_decl_start].a);
 
    /* Now scan for commas */
@@ -1096,6 +1163,91 @@ static void newline_func_def(chunk_t *start)
 }
 
 
+/**
+ * Checks to see if it is OK to add a newline around the chunk.
+ * Don't want to break one-liners...
+ */
+static bool one_liner_nl_ok(chunk_t *pc)
+{
+   LOG_FMT(LNL1LINE, "%s: check [%s] parent=[%s] flg=%x, on line %d, col %d - ",
+           __func__, get_token_name(pc->type), get_token_name(pc->parent_type),
+           pc->flags, pc->orig_line, pc->orig_col);
+
+   if ((pc->flags & PCF_ONE_LINER) &&
+       ((pc->type == CT_BRACE_OPEN) ||
+        (pc->type == CT_BRACE_CLOSE)))
+   {
+      if (cpd.settings[UO_nl_class_leave_one_liners].b &&
+          (pc->flags & PCF_IN_CLASS))
+      {
+         LOG_FMT(LNL1LINE, "false (class)\n");
+         return(false);
+      }
+
+      if (cpd.settings[UO_nl_assign_leave_one_liners].b &&
+          (pc->parent_type == CT_ASSIGN))
+      {
+         LOG_FMT(LNL1LINE, "false (assign)\n");
+         return(false);
+      }
+
+      if (cpd.settings[UO_nl_enum_leave_one_liners].b &&
+          (pc->parent_type == CT_ENUM))
+      {
+         LOG_FMT(LNL1LINE, "false (enum)\n");
+         return(false);
+      }
+
+      if (cpd.settings[UO_nl_getset_leave_one_liners].b &&
+          (pc->parent_type == CT_GETSET))
+      {
+         LOG_FMT(LNL1LINE, "false (get/set)\n");
+         return(false);
+      }
+
+      if (cpd.settings[UO_nl_func_leave_one_liners].b &&
+          (pc->parent_type == CT_FUNC_DEF))
+      {
+         LOG_FMT(LNL1LINE, "false (func def)\n");
+         return(false);
+      }
+
+      if (cpd.settings[UO_nl_if_leave_one_liners].b &&
+          ((pc->parent_type == CT_IF) ||
+           (pc->parent_type == CT_ELSE)))
+      {
+         LOG_FMT(LNL1LINE, "false (if/else)\n");
+         return(false);
+      }
+   }
+   LOG_FMT(LNL1LINE, "true\n");
+   return(true);
+}
+
+static void nl_create_one_liner(chunk_t *vbrace_open)
+{
+   chunk_t *tmp;
+   chunk_t *first;
+
+   /* See if we get a newline between the next text and the vbrace_close */
+   tmp   = chunk_get_next_ncnl(vbrace_open);
+   first = tmp;
+   while ((tmp != NULL) && (tmp->type != CT_VBRACE_CLOSE))
+   {
+      if (chunk_is_newline(tmp))
+      {
+         return;
+      }
+      tmp = chunk_get_next(tmp);
+   }
+
+   if (tmp != NULL)
+   {
+      newline_del_between(vbrace_open, first);
+   }
+}
+
+
 /**
  * Step through all chunks.
  */
@@ -1103,6 +1255,8 @@ void newlines_cleanup_braces(void)
 {
    chunk_t  *pc;
    chunk_t  *next;
+   chunk_t  *prev;
+   chunk_t  *tmp;
    argval_t arg;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
@@ -1170,27 +1324,136 @@ void newlines_cleanup_braces(void)
       }
       else if (pc->type == CT_BRACE_OPEN)
       {
-         next = chunk_get_next_ncnl(pc);
+         if (cpd.settings[UO_nl_ds_struct_enum_cmt].b &&
+             ((pc->parent_type == CT_ENUM) ||
+              (pc->parent_type == CT_STRUCT) ||
+              (pc->parent_type == CT_UNION)))
+         {
+            newlines_double_space_struct_enum_union(pc);
+         }
+
+         if ((pc->parent_type == CT_CLASS) && (pc->level == pc->brace_level))
+         {
+            newlines_do_else(chunk_get_prev(pc),
+                             cpd.settings[UO_nl_class_brace].a);
+         }
+
+         next = chunk_get_next_nnl(pc);
          if (next->type == CT_BRACE_CLOSE)
          {
             //TODO: add an option to split open empty statements? { };
          }
+         else if (next->type == CT_BRACE_OPEN)
+         {
+            //TODO: do something with two brace opens on the same line? "{ {"
+         }
          else
          {
+            next = chunk_get_next_ncnl(pc);
+
+            // Handle nl_after_brace_open
             if ((pc->level == pc->brace_level) &&
-                cpd.settings[UO_nl_after_brace_open].b &&
-                ((pc->flags & (PCF_IN_ARRAY_ASSIGN | PCF_IN_PREPROC)) == 0) &&
-                ((pc->flags & PCF_ONE_CLASS) != PCF_ONE_CLASS))
+                cpd.settings[UO_nl_after_brace_open].b)
             {
-               if (!chunk_is_str(next, "{", 1))
+               if (!one_liner_nl_ok(pc))
                {
-                  newline_iarf(pc, AV_ADD);
+                  /* no change - preserve one liner body */
+               }
+               else if ((pc->flags & (PCF_IN_ARRAY_ASSIGN | PCF_IN_PREPROC)) != 0)
+               {
+                  /* no change - don't break up array assignments or preprocessors */
+               }
+               else
+               {
+                  /* Step back from next to the first non-newline item */
+                  tmp = chunk_get_prev(next);
+                  while (tmp != pc)
+                  {
+                     if (chunk_is_comment(tmp))
+                     {
+                        if (!cpd.settings[UO_nl_after_brace_open_cmt].b &&
+                            (tmp->type != CT_COMMENT_MULTI))
+                        {
+                           break;
+                        }
+                     }
+                     tmp = chunk_get_prev(tmp);
+                  }
+                  /* Add the newline */
+                  newline_iarf(tmp, AV_ADD);
                }
             }
          }
 
          newlines_brace_pair(pc);
       }
+      else if (pc->type == CT_BRACE_CLOSE)
+      {
+         if (cpd.settings[UO_eat_blanks_before_close_brace].b)
+         {
+            /* Limit the newlines before the close brace to 1 */
+            prev = chunk_get_prev(pc);
+            if (chunk_is_newline(prev))
+            {
+               prev->nl_count = 1;
+            }
+         }
+         else if (cpd.settings[UO_nl_ds_struct_enum_close_brace].b &&
+                  ((pc->parent_type == CT_ENUM) ||
+                   (pc->parent_type == CT_STRUCT) ||
+                   (pc->parent_type == CT_UNION)))
+         {
+            if ((pc->flags & PCF_ONE_LINER) == 0)
+            {
+               /* Make sure the } is preceeded by two newlines */
+               prev = chunk_get_prev(pc);
+               if (!chunk_is_newline(prev))
+               {
+                  prev = newline_add_before(pc);
+               }
+               if (prev->nl_count < 2)
+               {
+                  prev->nl_count = 2;
+               }
+            }
+         }
+
+         /* Force a newline after a function def */
+         if (pc->parent_type == CT_FUNC_DEF)
+         {
+            next = chunk_get_next(pc);
+            if ((next != NULL) &&
+                (next->type != CT_SEMICOLON) &&
+                !chunk_is_newline(next) &&
+                !chunk_is_comment(next))
+            {
+               newline_iarf(pc, AV_ADD);
+            }
+         }
+      }
+      else if (pc->type == CT_VBRACE_OPEN)
+      {
+         if (cpd.settings[UO_nl_after_vbrace_open].b)
+         {
+            next = chunk_get_next(pc);
+            if ((next->type != CT_VBRACE_CLOSE) &&
+                !chunk_is_comment(next) &&
+                !chunk_is_newline(next))
+            {
+               newline_iarf(pc, AV_ADD);
+            }
+         }
+
+         if (((pc->parent_type == CT_IF) &&
+              cpd.settings[UO_nl_create_if_one_liner].b) ||
+             ((pc->parent_type == CT_FOR) &&
+              cpd.settings[UO_nl_create_for_one_liner].b) ||
+             ((pc->parent_type == CT_WHILE) &&
+              cpd.settings[UO_nl_create_while_one_liner].b))
+         {
+            nl_create_one_liner(pc);
+         }
+      }
       else if (pc->type == CT_STRUCT)
       {
          newlines_struct_enum_union(pc, cpd.settings[UO_nl_struct_brace].a);
@@ -1223,14 +1486,11 @@ void newlines_cleanup_braces(void)
          next = chunk_get_next_ncnl(pc);
          if ((next != NULL) && (next->type == CT_BRACE_OPEN))
          {
-            if ((cpd.settings[UO_nl_fcall_brace].a & AV_ADD) != 0)
-            {
-               /*TODO: insert a brace? */
-            }
-            else if ((cpd.settings[UO_nl_fcall_brace].a & AV_REMOVE) != 0)
-            {
-               /*TODO: insert a brace? */
-            }
+            /* TODO:
+             * this could be used to control newlines between the
+             * the if/while/for/switch close paren and the open brace, but
+             * that is currently handled elsewhere.
+             */
          }
       }
       else if (pc->type == CT_RETURN)
@@ -1246,9 +1506,17 @@ void newlines_cleanup_braces(void)
              cpd.settings[UO_nl_after_semicolon].b)
          {
             next = chunk_get_next(pc);
-            if (!chunk_is_comment(next))
+            while ((next != NULL) && (next->type == CT_VBRACE_CLOSE))
             {
-               newline_iarf(pc, AV_ADD);
+               next = chunk_get_next(next);
+            }
+            if (!chunk_is_comment(next) &&
+                !chunk_is_newline(next))
+            {
+               if (one_liner_nl_ok(next))
+               {
+                  newline_iarf(pc, AV_ADD);
+               }
             }
          }
       }
@@ -1260,18 +1528,12 @@ void newlines_cleanup_braces(void)
              ((cpd.settings[UO_nl_func_decl_start].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_args].a != AV_IGNORE) ||
               (cpd.settings[UO_nl_func_decl_end].a != AV_IGNORE) ||
-              (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE)))
+              (cpd.settings[UO_nl_func_type_name].a != AV_IGNORE) ||
+              (cpd.settings[UO_nl_func_paren].a != AV_IGNORE)))
          {
             newline_func_def(pc);
          }
       }
-      else if (pc->type == CT_CLASS)
-      {
-         if (pc->level == pc->brace_level)
-         {
-            newlines_struct_enum_union(pc, cpd.settings[UO_nl_class_brace].a);
-         }
-      }
       else if (pc->type == CT_ANGLE_CLOSE)
       {
          if (pc->parent_type == CT_TEMPLATE)
@@ -1288,6 +1550,59 @@ void newlines_cleanup_braces(void)
       {
          newlines_struct_enum_union(pc, cpd.settings[UO_nl_namespace_brace].a);
       }
+      else if (pc->type == CT_SQUARE_OPEN)
+      {
+         if ((pc->parent_type == CT_ASSIGN) &&
+             ((pc->flags & PCF_ONE_LINER) == 0))
+         {
+            tmp = chunk_get_prev_ncnl(pc);
+            newline_iarf(tmp, cpd.settings[UO_nl_assign_square].a);
+
+            argval_t arg = cpd.settings[UO_nl_after_square_assign].a;
+
+            if (cpd.settings[UO_nl_assign_square].a & AV_ADD)
+            {
+               arg = AV_ADD;
+            }
+            newline_iarf(pc, arg);
+
+            /* if there is a newline after the open, then force a newline
+             * before the close */
+            tmp = chunk_get_next_nc(pc);
+            if (chunk_is_newline(tmp))
+            {
+               tmp = chunk_get_next_type(pc, CT_SQUARE_CLOSE, pc->level);
+               if (tmp != NULL)
+               {
+                  newline_add_before(tmp);
+               }
+            }
+         }
+      }
+      else if (pc->type == CT_PRIVATE)
+      {
+         /** Make sure there is a newline before an access spec */
+         if (cpd.settings[UO_nl_before_access_spec].n > 0)
+         {
+            prev = chunk_get_prev(pc);
+            if (!chunk_is_newline(prev))
+            {
+               newline_add_before(pc);
+            }
+         }
+      }
+      else if (pc->type == CT_PRIVATE_COLON)
+      {
+         /** Make sure there is a newline after an access spec */
+         if (cpd.settings[UO_nl_after_access_spec].n > 0)
+         {
+            next = chunk_get_next(pc);
+            if (!chunk_is_newline(next))
+            {
+               newline_add_before(next);
+            }
+         }
+      }
       else
       {
          /* ignore it */
@@ -1296,6 +1611,30 @@ void newlines_cleanup_braces(void)
 }
 
 
+void newline_after_multiline_comment(void)
+{
+   chunk_t *pc;
+   chunk_t *tmp;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if (pc->type != CT_COMMENT_MULTI)
+      {
+         continue;
+      }
+
+      tmp = pc;
+      while (((tmp = chunk_get_next(tmp)) != NULL) && !chunk_is_newline(tmp))
+      {
+         if (!chunk_is_comment(tmp))
+         {
+            newline_add_before(tmp);
+            break;
+         }
+      }
+   }
+}
+
 /**
  * Handle insertion/removal of blank lines before if/for/while/do
  */
@@ -1337,7 +1676,6 @@ void newlines_insert_blank_lines(void)
    }
 }
 
-
 void newlines_squeeze_ifdef(void)
 {
    chunk_t *pc;
@@ -1392,7 +1730,6 @@ void newlines_squeeze_ifdef(void)
    }
 }
 
-
 void newlines_eat_start_end(void)
 {
    chunk_t *pc;
@@ -1464,19 +1801,16 @@ void newlines_eat_start_end(void)
    }
 }
 
-
 /**
- * Searches for CT_BOOL (|| or && or ^^) operators and moves them, if needed.
- * Will not move CT_BOOL tokens that are on their own line or have other than
- * exactly 1 newline before (UO_pos_bool == TRAIL) or
- * after (UO_pos_bool == LEAD).
+ * Searches for a chunk of type chunk_type and moves them, if needed.
+ * Will not move tokens that are on their own line or have other than
+ * exactly 1 newline before (UO_pos_comma == TRAIL) or after (UO_pos_comma == LEAD).
  */
-void newlines_bool_pos(void)
+void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
 {
-   chunk_t    *pc;
-   chunk_t    *next;
-   chunk_t    *prev;
-   tokenpos_e mode = cpd.settings[UO_pos_bool].tp;
+   chunk_t *pc;
+   chunk_t *next;
+   chunk_t *prev;
 
    if (mode == TP_IGNORE)
    {
@@ -1485,7 +1819,7 @@ void newlines_bool_pos(void)
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
    {
-      if (pc->type == CT_BOOL)
+      if (pc->type == chunk_type)
       {
          prev = chunk_get_prev_nc(pc);
          next = chunk_get_next_nc(pc);
@@ -1522,7 +1856,6 @@ void newlines_bool_pos(void)
    }
 }
 
-
 /**
  * Searches for CT_CLASS_COLON and moves them, if needed.
  * Also breaks up the args
@@ -1549,21 +1882,23 @@ void newlines_class_colon_pos(void)
          next   = chunk_get_next_nc(pc);
 
          if (!chunk_is_newline(prev) && !chunk_is_newline(next) &&
-             ((cpd.settings[UO_nl_class_init_args].a & AV_ADD) != 0))
+             ((cpd.settings[UO_nl_class_colon].a & AV_ADD) != 0))
          {
             newline_add_after(pc);
             prev = chunk_get_prev_nc(pc);
             next = chunk_get_next_nc(pc);
          }
 
-         if (cpd.settings[UO_nl_class_init_args].a == AV_REMOVE)
+         if (cpd.settings[UO_nl_class_colon].a == AV_REMOVE)
          {
-            if (chunk_is_newline(prev))
+            if (chunk_is_newline(prev) &&
+                chunk_safe_to_del_nl(prev))
             {
                chunk_del(prev);
                prev = chunk_get_prev_nc(pc);
             }
-            if (chunk_is_newline(next))
+            if (chunk_is_newline(next) &&
+                chunk_safe_to_del_nl(next))
             {
                chunk_del(next);
                next = chunk_get_next_nc(pc);
@@ -1597,12 +1932,24 @@ void newlines_class_colon_pos(void)
          {
             if ((cpd.settings[UO_nl_class_init_args].a & AV_ADD) != 0)
             {
-               newline_add_after(pc);
+               if (cpd.settings[UO_pos_class_comma].tp == TP_TRAIL)
+               {
+                  newline_add_after(pc);
+               }
+               else
+               {
+                  newline_add_before(pc);
+                  next = chunk_get_next_nc(pc);
+                  if (chunk_is_newline(next) && chunk_safe_to_del_nl(next))
+                  {
+                     chunk_del(next);
+                  }
+               }
             }
             else if (cpd.settings[UO_nl_class_init_args].a == AV_REMOVE)
             {
                next = chunk_get_next(pc);
-               if ((next != NULL) && (next->type == CT_NEWLINE))
+               if (chunk_is_newline(next) && chunk_safe_to_del_nl(next))
                {
                   chunk_del(next);
                }
@@ -1612,7 +1959,6 @@ void newlines_class_colon_pos(void)
    }
 }
 
-
 /**
  * Scans for newline tokens and limits the nl_count.
  * A newline token has a minimum nl_count of 1.
@@ -1695,9 +2041,37 @@ void do_blank_lines(void)
          }
       }
 
+      /** Control blanks before an access spec */
+      if ((cpd.settings[UO_nl_before_access_spec].n > 0) &&
+          (cpd.settings[UO_nl_before_access_spec].n != pc->nl_count) &&
+          (next != NULL) &&
+          (next->type == CT_PRIVATE))
+      {
+         /* Don't add blanks after a open brace */
+         if ((prev == NULL) ||
+             ((prev->type != CT_BRACE_OPEN) &&
+              (prev->type != CT_VBRACE_OPEN)))
+         {
+            LOG_FMT(LCMTNL, "%s: NL-BeforePrivate: line %d\n", __func__, next->orig_line);
+            pc->nl_count = cpd.settings[UO_nl_before_access_spec].n;
+         }
+      }
+
+      /** Control blanks after an access spec */
+      if ((cpd.settings[UO_nl_after_access_spec].n > 0) &&
+          (cpd.settings[UO_nl_after_access_spec].n != pc->nl_count) &&
+          (prev != NULL) &&
+          (prev->type == CT_PRIVATE_COLON))
+      {
+         LOG_FMT(LCMTNL, "%s: NL-AfterPrivate: line %d\n", __func__, prev->orig_line);
+         pc->nl_count = cpd.settings[UO_nl_after_access_spec].n;
+      }
+
       /* Add blanks after function bodies */
       if ((prev != NULL) && (prev->type == CT_BRACE_CLOSE) &&
-          ((prev->parent_type == CT_FUNC_DEF) || (prev->parent_type == CT_FUNC_CLASS)))
+          ((prev->parent_type == CT_FUNC_DEF) ||
+           (prev->parent_type == CT_FUNC_CLASS) ||
+           (prev->parent_type == CT_ASSIGN)))
       {
          if (prev->flags & PCF_ONE_LINER)
          {
@@ -1708,7 +2082,8 @@ void do_blank_lines(void)
          }
          else
          {
-            if (cpd.settings[UO_nl_after_func_body].n > pc->nl_count)
+            if ((cpd.settings[UO_nl_after_func_body].n > 0) &&
+                (cpd.settings[UO_nl_after_func_body].n != pc->nl_count))
             {
                pc->nl_count = cpd.settings[UO_nl_after_func_body].n;
             }
@@ -1756,83 +2131,36 @@ void newlines_cleanup_dup(void)
 }
 
 
-void newlines_double_space_struct_enum_union(void)
+/**
+ * Make sure there is a blank line after a commented group of values
+ */
+static void newlines_double_space_struct_enum_union(chunk_t *open_brace)
 {
-   chunk_t *pc;
-   chunk_t *next;
+   chunk_t *pc = open_brace;
    chunk_t *prev;
-   bool inside_seu = false;
-   int seu_level = 0;
 
-   prev = NULL;
-   pc   = chunk_get_head();
-
-   while (pc != NULL)
+   while (((pc = chunk_get_next_nc(pc)) != NULL) &&
+          (pc->level > open_brace->level))
    {
-      next = chunk_get_next(pc);
-
-      if (!inside_seu)
+      if ((pc->level != (open_brace->level + 1)) ||
+          (pc->type != CT_NEWLINE))
       {
-         if ((pc->type == CT_BRACE_OPEN) &&
-             ((pc->parent_type == CT_STRUCT) ||
-              (pc->parent_type == CT_ENUM) ||
-              (pc->parent_type == CT_UNION)))
-         {
-            inside_seu = true;
-            seu_level = pc->brace_level;
-         }
-      }
-      else
-      {
-         if (pc->type == CT_NEWLINE)
-         {
-            switch (prev->type)
-            {
-            case CT_COMMENT_WHOLE:
-            case CT_COMMENT_MULTI:
-            case CT_BRACE_OPEN:
-            case CT_COMMENT_CPP:
-            case CT_COMMENT:
-               break;
-
-            default:
-               switch (next->type)
-               {
-               case CT_COMMENT_WHOLE:
-               case CT_COMMENT_MULTI:
-               case CT_COMMENT_CPP:
-               case CT_COMMENT:
-                  if (pc->nl_count < 2)
-                  {
-                     pc->nl_count = 2;
-                  }
-                  break;
-
-               default:
-                  break;
-               }
-               break;
-            }
-         }
-         else if (pc->type == CT_BRACE_CLOSE)
-         {
-            if (prev->type != CT_NEWLINE)
-            {
-               prev = newline_add_before(pc);
-            }
-            if (prev->nl_count < 2)
-            {
-               prev->nl_count = 2;
-            }
-         }
-
-         if ((pc->type == CT_BRACE_CLOSE) && (pc->brace_level == seu_level))
-         {
-            inside_seu = false;
-         }
+         continue;
       }
 
-      prev = pc;
-      pc   = next;
+      /* If the newline is NOT after a comment or a brace open and
+       * it is before a comment, then make sure that the newline is
+       * at least doubled
+       */
+      prev = chunk_get_prev(pc);
+      if (!chunk_is_comment(prev) &&
+          (prev->type != CT_BRACE_OPEN) &&
+          chunk_is_comment(chunk_get_next(pc)))
+      {
+         if (pc->nl_count < 2)
+         {
+            pc->nl_count = 2;
+         }
+      }
    }
 }
