@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: parse_frame.cpp 611 2006-12-20 04:12:46Z bengardner $
+ * $Id: parse_frame.cpp 987 2007-11-02 18:53:04Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -34,7 +34,6 @@ void pf_log(int logsev, struct parse_frame *pf)
    LOG_FMT(logsev, "\n");
 }
 
-
 /**
  * Logs the entire parse frame stack
  */
@@ -60,7 +59,6 @@ void pf_copy(struct parse_frame *dst, const struct parse_frame *src)
    memcpy(dst, src, sizeof(struct parse_frame));
 }
 
-
 /**
  * Push a copy of the parse frame onto the stack.
  * This is called on #if and #ifdef.
@@ -75,7 +73,6 @@ void pf_push(struct parse_frame *pf)
    LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
 }
 
-
 /**
  * Push a copy of the parse frame onto the stack, under the tos.
  * If this were a linked list, just add before the last item.
@@ -101,7 +98,6 @@ void pf_push_under(struct parse_frame *pf)
    LOG_FMT(LPF, "%s: after count = %d\n", __func__, cpd.frame_count);
 }
 
-
 /**
  * Copy the top item off the stack into pf.
  * This is called on #else and #elif.
@@ -115,7 +111,6 @@ void pf_copy_tos(struct parse_frame *pf)
    LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
 }
 
-
 /**
  * Deletes the top frame from the stack.
  */
@@ -128,7 +123,6 @@ void pf_trash_tos(void)
    LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
 }
 
-
 /**
  * Pop the top item off the stack and copy into pf.
  * This is called on #endif
@@ -153,19 +147,27 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
    int        pp_level = cpd.pp_level;
    const char *txt     = NULL;
 
+   if (pc->type != CT_PREPROC)
+   {
+      return(pp_level);
+   }
+
+   LOG_FMT(LPFCHK, "%s: %5d] %s\n",
+           __func__, pc->orig_line, get_token_name(pc->parent_type));
+
    if ((pc->flags & PCF_IN_PREPROC) != 0)
    {
       LOG_FMT(LPF, " <In> ");
       pf_log(LPF, frm);
 
-      if (pc->type == CT_PP_IF)
+      if (pc->parent_type == CT_PP_IF)
       {
          cpd.pp_level++;
          pf_push(frm);
          frm->in_ifdef = CT_PP_IF;
          txt           = "if-push";
       }
-      else if (pc->type == CT_PP_ELSE)
+      else if (pc->parent_type == CT_PP_ELSE)
       {
          pp_level--;
          if (frm->in_ifdef == CT_PP_IF)
@@ -187,7 +189,7 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
             txt = "???";
          }
       }
-      else if (pc->type == CT_PP_ENDIF)
+      else if (pc->parent_type == CT_PP_ENDIF)
       {
          cpd.pp_level--;
          pp_level--;
@@ -213,7 +215,7 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
    if (txt != NULL)
    {
       LOG_FMT(LPF, "%s: %d> %s: %s in_ifdef=%d/%d counts=%d/%d\n", __func__,
-              pc->orig_line, get_token_name(pc->type), txt,
+              pc->orig_line, get_token_name(pc->parent_type), txt,
               in_ifdef, frm->in_ifdef, b4_cnt, cpd.frame_count);
       pf_log_all(LPF);
       LOG_FMT(LPF, " <Out>");
