@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: prototypes.h 705 2007-03-04 22:16:24Z bengardner $
+ * $Id: prototypes.h 984 2007-10-31 01:10:41Z bengardner $
  */
 #ifndef C_PARSE_PROTOTYPES_H_INCLUDED
 #define C_PARSE_PROTOTYPES_H_INCLUDED
@@ -21,7 +21,8 @@
 const char *get_token_name(c_token_t token);
 void log_pcf_flags(log_sev_t sev, UINT32 flags);
 const char *path_basename(const char *path);
-const char *get_file_extension(int &idx);
+int path_dirname_len(const char *filename);
+const char *get_file_extension(int& idx);
 
 
 /*
@@ -31,7 +32,6 @@ const char *get_file_extension(int &idx);
 void output_text(FILE *pfile);
 void output_parsed(FILE *pfile);
 void output_comment_multi(chunk_t *pc);
-chunk_t *output_comment_cpp(chunk_t *pc);
 void output_options(FILE *pfile);
 void output_to_column(int column, bool allow_tabs);
 void add_text(const char *text);
@@ -68,7 +68,8 @@ std::string op_val_to_string(argtype_e argtype, op_val_t op_val);
 void indent_text(void);
 void indent_preproc(void);
 void indent_to_column(chunk_t *pc, int column);
-void reindent_line(chunk_t *pc, int column);
+#define reindent_line(pc, col)   reindent_line2(pc, col, __func__, __LINE__)
+void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno);
 
 
 /*
@@ -120,6 +121,12 @@ void combine_labels(void);
 void mark_comments(void);
 void make_type(chunk_t *pc);
 
+chunk_t *skip_template_next(chunk_t *ang_open);
+chunk_t *skip_template_prev(chunk_t *ang_close);
+
+chunk_t *skip_attribute_next(chunk_t *attr);
+chunk_t *skip_attribute_prev(chunk_t *fp_close);
+
 
 /*
  *  newlines.cpp
@@ -129,10 +136,10 @@ void newlines_cleanup_braces(void);
 void newlines_insert_blank_lines(void);
 void newlines_squeeze_ifdef(void);
 void newlines_eat_start_end(void);
-void newlines_bool_pos(void);
+void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode);
 void newlines_class_colon_pos(void);
 void newlines_cleanup_dup(void);
-void newlines_double_space_struct_enum_union(void);
+void newline_after_multiline_comment(void);
 void do_blank_lines(void);
 chunk_t *newline_add_before(chunk_t *pc);
 chunk_t *newline_add_after(chunk_t *pc);
@@ -142,7 +149,7 @@ chunk_t *newline_add_after(chunk_t *pc);
  *  tokenize.cpp
  */
 
-void tokenize(const char *data, int data_len);
+void tokenize(const char *data, int data_len, chunk_t *ref);
 
 
 /*
@@ -167,9 +174,10 @@ int load_keyword_file(const char *filename);
 const chunk_tag_t *find_keyword(const char *word, int len);
 void add_keyword(const char *tag, c_token_t type, UINT8 lang_flags);
 void output_types(FILE *pfile);
-const chunk_tag_t *get_custom_keyword_idx(int &idx);
+const chunk_tag_t *get_custom_keyword_idx(int& idx);
 void clear_keyword_file(void);
 pattern_class get_token_pattern_class(c_token_t tok);
+bool keywords_are_sorted(void);
 
 
 /*
@@ -179,7 +187,7 @@ pattern_class get_token_pattern_class(c_token_t tok);
 int load_define_file(const char *filename);
 const define_tag_t *find_define(const char *word, int len);
 void add_define(const char *tag, const char *value);
-const define_tag_t *get_define_idx(int &idx);
+const define_tag_t *get_define_idx(int& idx);
 void output_defines(FILE *pfile);
 void clear_defines(void);
 
@@ -215,6 +223,8 @@ void do_code_width(void);
 void pawn_prescan(void);
 void pawn_add_virtual_semicolons();
 chunk_t *pawn_check_vsemicolon(chunk_t *pc);
+void pawn_scrub_vsemi(void);
+chunk_t *pawn_add_vsemi_after(chunk_t *pc);
 
 
 /*
@@ -223,6 +233,12 @@ chunk_t *pawn_check_vsemicolon(chunk_t *pc);
 void print_universal_indent_cfg(FILE *pfile);
 
 
+/*
+ * semicolons.cpp
+ */
+void remove_extra_semicolons(void);
+
+
 /**
  * Advances to the next tab stop.
  * Column 1 is the left-most column.
@@ -270,5 +286,4 @@ int align_tab_column(int col)
    return(col);
 }
 
-
 #endif   /* C_PARSE_PROTOTYPES_H_INCLUDED */
