@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: space.cpp 708 2007-03-06 02:16:16Z bengardner $
+ * $Id: space.cpp 972 2007-10-17 21:54:41Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -30,45 +30,42 @@ struct no_space_table_s
  */
 struct no_space_table_s no_space_table[] =
 {
-   { CT_INCDEC_BEFORE, CT_WORD          },
-   { CT_UNKNOWN,       CT_INCDEC_AFTER  },
-   { CT_UNKNOWN,       CT_ELLIPSIS      },
-   { CT_UNKNOWN,       CT_LABEL_COLON   },
-   { CT_UNKNOWN,       CT_PRIVATE_COLON },
-   { CT_UNKNOWN,       CT_SEMICOLON     },
-   { CT_UNKNOWN,       CT_D_TEMPLATE    },
-   { CT_D_TEMPLATE,    CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_MEMBER        },
-   { CT_MEMBER,        CT_UNKNOWN       },
-   { CT_MACRO_FUNC,    CT_FPAREN_OPEN   },
-   { CT_PAREN_OPEN,    CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_PAREN_CLOSE   },
-   { CT_FPAREN_OPEN,   CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_SPAREN_CLOSE  },
-   { CT_SPAREN_OPEN,   CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_FPAREN_CLOSE  },
-   { CT_UNKNOWN,       CT_COMMA         },
-   { CT_POS,           CT_UNKNOWN       },
-   { CT_ADDR,          CT_UNKNOWN       },
-   { CT_STAR,          CT_UNKNOWN       },
-   { CT_DEREF,         CT_UNKNOWN       },
-   { CT_NOT,           CT_UNKNOWN       },
-   { CT_INV,           CT_UNKNOWN       },
-   { CT_VBRACE_CLOSE,  CT_UNKNOWN       },
-   { CT_VBRACE_OPEN,   CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_VBRACE_CLOSE  },
-   { CT_UNKNOWN,       CT_VBRACE_OPEN   },
-   { CT_PREPROC,       CT_UNKNOWN       },
-   { CT_NEG,           CT_UNKNOWN       },
-   { CT_UNKNOWN,       CT_SQUARE_OPEN   },
-   { CT_UNKNOWN,       CT_SQUARE_CLOSE  },
-   { CT_UNKNOWN,       CT_CASE_COLON    },
-   { CT_SQUARE_OPEN,   CT_UNKNOWN       },
-   { CT_PAREN_CLOSE,   CT_WORD          },
-   { CT_PAREN_CLOSE,   CT_FUNC_DEF      },
-   { CT_PAREN_CLOSE,   CT_FUNC_CALL     },
-   { CT_PAREN_CLOSE,   CT_ADDR          },
-   { CT_PAREN_CLOSE,   CT_FPAREN_OPEN   },
+   { CT_INCDEC_BEFORE,  CT_WORD          },
+   { CT_UNKNOWN,        CT_INCDEC_AFTER  },
+   { CT_UNKNOWN,        CT_ELLIPSIS      },
+   { CT_UNKNOWN,        CT_LABEL_COLON   },
+   { CT_UNKNOWN,        CT_PRIVATE_COLON },
+   { CT_UNKNOWN,        CT_SEMICOLON     },
+   { CT_UNKNOWN,        CT_D_TEMPLATE    },
+   { CT_D_TEMPLATE,     CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_MEMBER        },
+   { CT_MEMBER,         CT_UNKNOWN       },
+   { CT_MACRO_FUNC,     CT_FPAREN_OPEN   },
+   { CT_PAREN_OPEN,     CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_PAREN_CLOSE   },
+   { CT_FPAREN_OPEN,    CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_SPAREN_CLOSE  },
+   { CT_SPAREN_OPEN,    CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_FPAREN_CLOSE  },
+   { CT_UNKNOWN,        CT_COMMA         },
+   { CT_POS,            CT_UNKNOWN       },
+   { CT_STAR,           CT_UNKNOWN       },
+   { CT_VBRACE_CLOSE,   CT_UNKNOWN       },
+   { CT_VBRACE_OPEN,    CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_VBRACE_CLOSE  },
+   { CT_UNKNOWN,        CT_VBRACE_OPEN   },
+   { CT_PREPROC,        CT_UNKNOWN       },
+   { CT_PREPROC_INDENT, CT_UNKNOWN       },
+   { CT_NEG,            CT_UNKNOWN       },
+   { CT_UNKNOWN,        CT_SQUARE_OPEN   },
+   { CT_UNKNOWN,        CT_SQUARE_CLOSE  },
+   { CT_UNKNOWN,        CT_CASE_COLON    },
+   { CT_SQUARE_OPEN,    CT_UNKNOWN       },
+   { CT_PAREN_CLOSE,    CT_WORD          },
+   { CT_PAREN_CLOSE,    CT_FUNC_DEF      },
+   { CT_PAREN_CLOSE,    CT_FUNC_CALL     },
+   { CT_PAREN_CLOSE,    CT_ADDR          },
+   { CT_PAREN_CLOSE,    CT_FPAREN_OPEN   },
 };
 
 
@@ -78,7 +75,7 @@ struct no_space_table_s no_space_table[] =
  *
  * @param first   The first chunk
  * @param second  The second chunk
- * @return -1=remove space, 0=no change, 1=add space, 2=force one space
+ * @return        AV_IGNORE, AV_ADD, AV_REMOVE or AV_FORCE
  */
 argval_t do_space(chunk_t *first, chunk_t *second)
 {
@@ -99,6 +96,12 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_REMOVE);
    }
 
+   if ((first->type == CT_D_ARRAY_COLON) ||
+       (second->type == CT_D_ARRAY_COLON))
+   {
+      return(cpd.settings[UO_sp_d_array_colon].a);
+   }
+
    if ((first->type == CT_COLON) && (first->parent_type == CT_SQL_EXEC))
    {
       return(AV_REMOVE);
@@ -144,10 +147,17 @@ argval_t do_space(chunk_t *first, chunk_t *second)
 
    if (second->type == CT_SEMICOLON)
    {
-      if ((second->parent_type == CT_FOR) &&
-          (cpd.settings[UO_sp_before_semi_for].a != AV_IGNORE))
+      if (second->parent_type == CT_FOR)
       {
-         return(cpd.settings[UO_sp_before_semi_for].a);
+         if ((cpd.settings[UO_sp_before_semi_for_empty].a != AV_IGNORE) &&
+             ((first->type == CT_SPAREN_OPEN) || (first->type == CT_SEMICOLON)))
+         {
+            return(cpd.settings[UO_sp_before_semi_for_empty].a);
+         }
+         if (cpd.settings[UO_sp_before_semi_for].a != AV_IGNORE)
+         {
+            return(cpd.settings[UO_sp_before_semi_for].a);
+         }
       }
 
       arg = cpd.settings[UO_sp_before_semi].a;
@@ -188,11 +198,14 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    }
 
    /* handle '::' */
-   if ((first->type == CT_DC_MEMBER) ||
-       ((second->type == CT_DC_MEMBER) &&
-        ((first->type == CT_WORD) || (first->type == CT_TYPE))))
+   if (first->type == CT_DC_MEMBER)
    {
-      return(AV_REMOVE);
+      return(cpd.settings[UO_sp_after_dc].a);
+   }
+   if ((second->type == CT_DC_MEMBER) &&
+       ((first->type == CT_WORD) || (first->type == CT_TYPE)))
+   {
+      return(cpd.settings[UO_sp_before_dc].a);
    }
 
    if (first->type == CT_TAG_COLON)
@@ -227,6 +240,11 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_before_sparen].a);
    }
 
+   if ((first->type == CT_ASSIGN) || (second->type == CT_ASSIGN))
+   {
+      return(cpd.settings[UO_sp_assign].a);
+   }
+
    /* "a [x]" vs "a[x]" */
    if (second->type == CT_SQUARE_OPEN)
    {
@@ -239,23 +257,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_before_squares].a);
    }
 
-   if (first->type == CT_BYREF)
-   {
-      return(cpd.settings[UO_sp_after_byref].a);
-   }
-
-   if (second->type == CT_BYREF)
-   {
-      return(cpd.settings[UO_sp_before_byref].a);
-   }
-
-   /* spacing around template > > stuff */
-   if ((first->type == CT_ANGLE_CLOSE) &&
-       (second->type == CT_ANGLE_CLOSE))
-   {
-      return(AV_FORCE);
-   }
-
    /* spacing around template < > stuff */
    if ((first->type == CT_ANGLE_OPEN) ||
        (second->type == CT_ANGLE_CLOSE))
@@ -268,7 +269,33 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    }
    if (first->type == CT_ANGLE_CLOSE)
    {
-      return(cpd.settings[UO_sp_after_angle].a);
+      if (second->type == CT_WORD)
+      {
+         return(cpd.settings[UO_sp_angle_word].a);
+      }
+      if ((second->type == CT_FPAREN_OPEN) ||
+          (second->type == CT_PAREN_OPEN))
+      {
+         return(cpd.settings[UO_sp_angle_paren].a);
+      }
+      if (second->type == CT_DC_MEMBER)
+      {
+         return(cpd.settings[UO_sp_before_dc].a);
+      }
+      if (second->type != CT_BYREF)
+      {
+         return(cpd.settings[UO_sp_after_angle].a);
+      }
+   }
+
+   if ((first->type == CT_BYREF) && CharTable::IsKw1(second->str[0]))
+   {
+      return(cpd.settings[UO_sp_after_byref].a);
+   }
+
+   if (second->type == CT_BYREF)
+   {
+      return(cpd.settings[UO_sp_before_byref].a);
    }
 
    /* "for (...) {...}" vs "for (...){...}" */
@@ -283,11 +310,17 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    {
       return(cpd.settings[UO_sp_func_call_paren].a);
    }
+   if (first->type == CT_ATTRIBUTE)
+   {
+      return(cpd.settings[UO_sp_attribute_paren].a);
+   }
    if (first->type == CT_FUNC_DEF)
    {
       return(cpd.settings[UO_sp_func_def_paren].a);
    }
-   if (first->type == CT_FUNC_PROTO)
+   if ((first->type == CT_FUNC_PROTO) ||
+       ((second->type == CT_FPAREN_OPEN) &&
+        (second->parent_type == CT_FUNC_PROTO)))
    {
       return(cpd.settings[UO_sp_func_proto_paren].a);
    }
@@ -300,20 +333,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_FORCE);
    }
 
-   if (first->type == CT_BRACE_OPEN)
-   {
-      if (first->parent_type == CT_ENUM)
-      {
-         return(cpd.settings[UO_sp_inside_braces_enum].a);
-      }
-      if ((first->parent_type == CT_STRUCT) ||
-          (first->parent_type == CT_UNION))
-      {
-         return(cpd.settings[UO_sp_inside_braces_struct].a);
-      }
-      return(cpd.settings[UO_sp_inside_braces].a);
-   }
-
    if (second->type == CT_BRACE_CLOSE)
    {
       if (second->parent_type == CT_ENUM)
@@ -328,13 +347,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_inside_braces].a);
    }
 
-   /* "a = { ... }" vs "a = {...}" */
-   if (((first->type == CT_BRACE_OPEN) && (first->parent_type == CT_ASSIGN)) ||
-       ((second->type == CT_BRACE_CLOSE) && (second->parent_type == CT_ASSIGN)))
-   {
-      return(cpd.settings[UO_sp_func_call_paren].a);
-   }
-
    if (first->type == CT_CAST)
    {
       return(AV_REMOVE);
@@ -390,31 +402,16 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_else_brace].a);
    }
 
-   if ((first->type == CT_BRACE_OPEN) && (second->type == CT_ELSE))
-   {
-      return(cpd.settings[UO_sp_brace_else].a);
-   }
-
    if ((first->type == CT_CATCH) && (second->type == CT_BRACE_OPEN))
    {
       return(cpd.settings[UO_sp_catch_brace].a);
    }
 
-   if ((first->type == CT_BRACE_OPEN) && (second->type == CT_CATCH))
-   {
-      return(cpd.settings[UO_sp_brace_catch].a);
-   }
-
    if ((first->type == CT_FINALLY) && (second->type == CT_BRACE_OPEN))
    {
       return(cpd.settings[UO_sp_finally_brace].a);
    }
 
-   if ((first->type == CT_BRACE_OPEN) && (second->type == CT_FINALLY))
-   {
-      return(cpd.settings[UO_sp_brace_finally].a);
-   }
-
    if ((first->type == CT_TRY) && (second->type == CT_BRACE_OPEN))
    {
       return(cpd.settings[UO_sp_try_brace].a);
@@ -468,8 +465,21 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    }
 
    /* "(a + 3)" vs "( a + 3 )" */
-   if ((first->type == CT_PAREN_OPEN) || (second->type == CT_PAREN_CLOSE))
+   if (first->type == CT_PAREN_OPEN)
    {
+      if (first->parent_type == CT_CAST)
+      {
+         return(cpd.settings[UO_sp_inside_paren_cast].a);
+      }
+      return(cpd.settings[UO_sp_inside_paren].a);
+   }
+
+   if (second->type == CT_PAREN_CLOSE)
+   {
+      if (second->parent_type == CT_CAST)
+      {
+         return(cpd.settings[UO_sp_inside_paren_cast].a);
+      }
       return(cpd.settings[UO_sp_inside_paren].a);
    }
 
@@ -483,6 +493,18 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_square_fparen].a);
    }
 
+   /* "for (;;)" vs "for (;; )" */
+   if ((second->type == CT_SPAREN_CLOSE) &&
+       (first->type == CT_SEMICOLON) &&
+       (second->parent_type == CT_FOR))
+   {
+      arg = cpd.settings[UO_sp_after_semi_for_empty].a;
+      if (arg != AV_IGNORE)
+      {
+         return(arg);
+      }
+   }
+
    /* "if(...)" vs "if( ... )" */
    if ((first->type == CT_SPAREN_OPEN) || (second->type == CT_SPAREN_CLOSE))
    {
@@ -527,10 +549,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    {
       return(cpd.settings[UO_sp_compare].a);
    }
-   if ((first->type == CT_ASSIGN) || (second->type == CT_ASSIGN))
-   {
-      return(cpd.settings[UO_sp_assign].a);
-   }
 
    if ((first->type == CT_PAREN_OPEN) && (second->type == CT_PTR_TYPE))
    {
@@ -545,7 +563,8 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    }
 
    if ((first->type == CT_PTR_TYPE) &&
-       (cpd.settings[UO_sp_after_ptr_star].a != AV_IGNORE))
+       (cpd.settings[UO_sp_after_ptr_star].a != AV_IGNORE) &&
+       CharTable::IsKw1(second->str[0]))
    {
       return(cpd.settings[UO_sp_after_ptr_star].a);
    }
@@ -556,14 +575,8 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_before_ptr_star].a);
    }
 
-   if ((first->type == CT_OPERATOR) &&
-       ((second->type == CT_FUNC_DEF) ||
-        (second->type == CT_FUNC_PROTO)))
+   if (first->type == CT_OPERATOR)
    {
-      if (get_char_table(second->str[0]) & CT_KW2)
-      {
-         return(AV_FORCE);
-      }
       return(cpd.settings[UO_sp_after_operator].a);
    }
 
@@ -578,7 +591,19 @@ argval_t do_space(chunk_t *first, chunk_t *second)
 
    if (first->type == CT_BRACE_OPEN)
    {
-      if (first->parent_type == CT_ENUM)
+      if (second->type == CT_ELSE)
+      {
+         return(cpd.settings[UO_sp_brace_else].a);
+      }
+      else if (second->type == CT_CATCH)
+      {
+         return(cpd.settings[UO_sp_brace_catch].a);
+      }
+      else if (second->type == CT_FINALLY)
+      {
+         return(cpd.settings[UO_sp_brace_finally].a);
+      }
+      else if (first->parent_type == CT_ENUM)
       {
          return(cpd.settings[UO_sp_inside_braces_enum].a);
       }
@@ -622,13 +647,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(cpd.settings[UO_sp_before_sparen].a);
    }
 
-   if ((second->type == CT_SPAREN_CLOSE) &&
-       (first->type == CT_SEMICOLON) &&
-       (second->parent_type == CT_FOR))
-   {
-      return(AV_ADD);
-   }
-
    if ((first->type == CT_SPAREN_CLOSE) &&
        (second->type == CT_SEMICOLON) &&
        (first->parent_type == CT_WHILE_OF_DO))
@@ -636,7 +654,8 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_REMOVE); /*TODO: does this need to be configured? */
    }
 
-   if ((first->type == CT_QUALIFIER) || (first->type == CT_TYPE))
+   if ((second->type != CT_PTR_TYPE) &&
+       ((first->type == CT_QUALIFIER) || (first->type == CT_TYPE)))
    {
       return(AV_FORCE);
    }
@@ -657,6 +676,29 @@ argval_t do_space(chunk_t *first, chunk_t *second)
       return(AV_IGNORE);
    }
 
+   /* If nothing claimed the PTR_TYPE, then return ignore */
+   if ((first->type == CT_PTR_TYPE) || (second->type == CT_PTR_TYPE))
+   {
+      return(AV_IGNORE);
+   }
+
+   if (first->type == CT_NOT)
+   {
+      return(cpd.settings[UO_sp_not].a);
+   }
+   if (first->type == CT_INV)
+   {
+      return(cpd.settings[UO_sp_inv].a);
+   }
+   if (first->type == CT_ADDR)
+   {
+      return(cpd.settings[UO_sp_addr].a);
+   }
+   if (first->type == CT_DEREF)
+   {
+      return(cpd.settings[UO_sp_deref].a);
+   }
+
    for (idx = 0; idx < (int)ARRAY_SIZE(no_space_table); idx++)
    {
       if (((no_space_table[idx].first == CT_UNKNOWN) ||
@@ -671,7 +713,6 @@ argval_t do_space(chunk_t *first, chunk_t *second)
    return(AV_ADD);
 }
 
-
 /**
  * Marches through the whole file and checks to see how many spaces should be
  * between two chunks
@@ -680,10 +721,16 @@ void space_text(void)
 {
    chunk_t *pc;
    chunk_t *next;
-   int     column = 1;
+   chunk_t *tmp;
+   int     column;
    int     delta;
 
    pc = chunk_get_head();
+   if (pc == NULL)
+   {
+      return;
+   }
+   column = pc->column;
    while (pc != NULL)
    {
       next = chunk_get_next(pc);
@@ -712,7 +759,59 @@ void space_text(void)
             column = pc->orig_col_end;
          }
 
-         switch (do_space(pc, next))
+         /**
+          * Apply a general safety check
+          * If the two chunks combined will tokenize differently, then we
+          * must force a space.
+          * Two chunks -- "()" and "[]" will always tokenize differently.
+          * They are always safe to not have a space after them.
+          */
+         pc->flags &= ~PCF_FORCE_SPACE;
+         if ((pc->len > 0) &&
+             !chunk_is_str(pc, "[]", 2) &&
+             !chunk_is_str(pc, "()", 2))
+         {
+            /* Find the next non-empty chunk on this line */
+            tmp = next;
+            while ((tmp != NULL) && (tmp->len == 0) && !chunk_is_newline(tmp))
+            {
+               tmp = chunk_get_next(tmp);
+            }
+            if ((tmp != NULL) && (tmp->len > 0))
+            {
+               bool kw1 = CharTable::IsKw2(pc->str[pc->len - 1]);
+               bool kw2 = CharTable::IsKw1(next->str[0]);
+               if (kw1 && kw2)
+               {
+                  /* back-to-back words need a space */
+                  pc->flags |= PCF_FORCE_SPACE;
+               }
+               else if (!kw1 && !kw2 && (pc->len < 4) && (next->len < 4))
+               {
+                  /* We aren't dealing with keywords. concat and try punctuators */
+                  char buf[9];
+                  memcpy(buf, pc->str, pc->len);
+                  memcpy(buf + pc->len, next->str, next->len);
+                  buf[pc->len + next->len] = 0;
+
+                  const chunk_tag_t *ct;
+                  ct = find_punctuator(buf, cpd.lang_flags);
+                  if ((ct != NULL) && ((int)strlen(ct->tag) != pc->len))
+                  {
+                     /* punctuator parsed to a different size.. */
+                     pc->flags |= PCF_FORCE_SPACE;
+                  }
+               }
+            }
+         }
+
+         int av = do_space(pc, next);
+         if (pc->flags & PCF_FORCE_SPACE)
+         {
+            av |= AV_ADD;
+         }
+
+         switch (av)
          {
          case AV_FORCE:
             /* add exactly one space */
@@ -752,7 +851,6 @@ void space_text(void)
    }
 }
 
-
 /**
  * Marches through the whole file and adds spaces around nested parens
  */
@@ -776,8 +874,9 @@ void space_text_balance_nested_parens(void)
       {
          /* insert a space between the two opening parens */
          space_add_after(first, 1);
+
          /* find the closing paren that matches the 'first' open paren and force
-            a space before it */
+          * a space before it */
          cur  = next;
          prev = cur;
          while ((cur = chunk_get_next(cur)) != NULL)
@@ -794,8 +893,9 @@ void space_text_balance_nested_parens(void)
       {
          /* insert a space between the two closing parens */
          space_add_after(first, 1);
+
          /* find the opening paren that matches the 'next' close paren and force
-            a space after it */
+          * a space after it */
          cur = first;
          while ((cur = chunk_get_prev(cur)) != NULL)
          {
@@ -811,7 +911,6 @@ void space_text_balance_nested_parens(void)
    }
 }
 
-
 /**
  * Calculates the column difference between two chunks.
  * The rules are bent a bit here, as AV_IGNORE and AV_ADD become AV_FORCE.
@@ -876,9 +975,10 @@ void space_add_after(chunk_t *pc, int count)
    sp.str         = "                "; // 16 spaces
    sp.len         = count;
    sp.level       = pc->level;
-   sp.brace_level = pc->level;
+   sp.brace_level = pc->brace_level;
    sp.pp_level    = pc->pp_level;
    sp.column      = pc->column + pc->len;
+   sp.orig_line   = pc->orig_line;
 
    chunk_add_after(&sp, pc);
 }
