@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: tokenize.cpp 656 2007-02-14 04:20:47Z bengardner $
+ * $Id: tokenize.cpp 971 2007-10-17 21:26:22Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -56,10 +56,19 @@ bool parse_comment(chunk_t *pc)
    if (pc->str[1] == '/')
    {
       pc->type = CT_COMMENT_CPP;
-      while ((pc->str[len] != '\n') &&
-             (pc->str[len] != '\r') &&
-             (pc->str[len] != 0))
+      while (true)
       {
+         while ((pc->str[len] != '\n') &&
+                (pc->str[len] != '\r') &&
+                (pc->str[len] != 0))
+         {
+            len++;
+         }
+
+         if (pc->str[len - 1] != '\\')
+         {
+            break;
+         }
          len++;
       }
    }
@@ -162,7 +171,6 @@ bool parse_comment(chunk_t *pc)
    return(true);
 }
 
-
 /**
  * Count the number of characters in the number.
  * The next bit of text starts with a number (0-9 or '.'), so it is a number.
@@ -323,7 +331,6 @@ static bool parse_number(chunk_t *pc)
    return(true);
 }
 
-
 /**
  * Count the number of characters in a quoted string.
  * The next bit of text starts with a quote char " or ' or <.
@@ -336,12 +343,13 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
 {
    bool escaped = 0;
    int  end_ch;
-   int  len         = quote_idx;
-   char escape_char = cpd.settings[UO_string_escape_char].n;
+   int  len          = quote_idx;
+   char escape_char  = cpd.settings[UO_string_escape_char].n;
+   char escape_char2 = cpd.settings[UO_string_escape_char2].n;
 
    pc->type = CT_STRING;
 
-   end_ch = get_char_table(pc->str[len]) & 0xff;
+   end_ch = CharTable::Get(pc->str[len]) & 0xff;
    len++;
 
    for ( /* nada */; pc->str[len] != 0; len++)
@@ -350,6 +358,11 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
       {
          cpd.column++;
          if (pc->str[len] == escape_char)
+         {
+            escaped = (escape_char != 0);
+         }
+         else if ((pc->str[len] == escape_char2) &&
+                  (pc->str[len + 1] == end_ch))
          {
             escaped = allow_escape;
          }
@@ -365,10 +378,6 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
             len++;
             break;
          }
-
-         /* TODO: detect a newline in the string -- for an error condition?
-          * Some languages allow newlines in strings.
-          */
       }
       else
       {
@@ -388,7 +397,6 @@ static bool parse_string(chunk_t *pc, int quote_idx, bool allow_escape)
    return(true);
 }
 
-
 /**
  * Literal string, ends with single "
  * Two "" don't end the string.
@@ -423,7 +431,6 @@ static bool parse_cs_string(chunk_t *pc)
    return(true);
 }
 
-
 /**
  * Count the number of characters in a word.
  * The first character is already valid for a keyword
@@ -436,8 +443,7 @@ bool parse_word(chunk_t *pc, bool skipcheck)
    int len = 1;
    const chunk_tag_t *tag;
 
-   while ((pc->str[len] < 127) &&
-          ((get_char_table(pc->str[len]) & CT_KW2) != 0))
+   while ((pc->str[len] < 127) && CharTable::IsKw2(pc->str[len]))
    {
       len++;
    }
@@ -473,7 +479,6 @@ bool parse_word(chunk_t *pc, bool skipcheck)
    return(true);
 }
 
-
 /**
  * Count the number of whitespace characters.
  *
@@ -543,7 +548,6 @@ bool parse_whitespace(chunk_t *pc)
    return(len != 0);
 }
 
-
 /**
  * Called when we hit a backslash.
  * If there is nothing but whitespace until the newline, then this is a
@@ -578,7 +582,6 @@ static bool parse_bs_newline(chunk_t *pc)
    return(false);
 }
 
-
 /**
  * Skips the next bit of whatever and returns the type of block.
  *
@@ -607,7 +610,7 @@ static bool parse_next(chunk_t *pc)
    pc->len       = 0;
    pc->type      = CT_NONE;
    pc->nl_count  = 0;
-
+   pc->flags     = 0;
 
    /**
     *  Parse whitespace
@@ -620,7 +623,8 @@ static bool parse_next(chunk_t *pc)
    /**
     *  Handle unknown/unhandled preprocessors
     */
-   if (cpd.in_preproc == CT_PP_OTHER)
+   if ((cpd.in_preproc > CT_PP_BODYCHUNK) &&
+       (cpd.in_preproc <= CT_PP_OTHER))
    {
       /* Chunk to a newline or comment */
       pc->type = CT_PREPROC_BODY;
@@ -629,9 +633,18 @@ static bool parse_next(chunk_t *pc)
       {
          char ch = pc->str[pc->len];
 
-         /* Bail on a non-escaped newline or on a C++ comment start */
-         if ((((ch == '\n') || (ch == '\r')) && (last != '\\')) ||
-             ((ch == '/') && (pc->str[pc->len + 1] == '/')))
+         if ((ch == '\n') || (ch == '\r'))
+         {
+            /* Back off if this is an escaped newline */
+            if (last == '\\')
+            {
+               pc->len--;
+            }
+            break;
+         }
+
+         /* Quit on a C++ comment start */
+         if ((ch == '/') && (pc->str[pc->len + 1] == '/'))
          {
             break;
          }
@@ -672,8 +685,7 @@ static bool parse_next(chunk_t *pc)
          parse_cs_string(pc);
          return(true);
       }
-      if (((get_char_table(pc->str[1]) & CT_KW1) != 0) &&
-          parse_word(pc, true))
+      if (CharTable::IsKw1(pc->str[1]) && parse_word(pc, true))
       {
          return(true);
       }
@@ -733,14 +745,13 @@ static bool parse_next(chunk_t *pc)
 
    /* Check for pawn/ObjectiveC identifiers */
    if ((*pc->str == '@') &&
-       ((get_char_table(pc->str[1]) & CT_KW2) != 0) &&
+       CharTable::IsKw2(pc->str[1]) &&
        parse_word(pc, false))
    {
       return(true);
    }
 
-   if (((get_char_table(*pc->str) & CT_KW1) != 0) &&
-       parse_word(pc, false))
+   if (CharTable::IsKw1(*pc->str) && parse_word(pc, false))
    {
       return(true);
    }
@@ -750,6 +761,7 @@ static bool parse_next(chunk_t *pc)
       pc->type    = punc->type;
       pc->len     = strlen(punc->tag);
       cpd.column += pc->len;
+      pc->flags  |= PCF_PUNCTUATOR;
       return(true);
    }
 
@@ -764,7 +776,6 @@ static bool parse_next(chunk_t *pc)
    return(true);
 }
 
-
 /**
  * This function parses or tokenizes the whole buffer into a list.
  * It has to do some tricks to parse preprocessors.
@@ -772,8 +783,11 @@ static bool parse_next(chunk_t *pc)
  * If output_text() were called immediately after, two things would happen:
  *  - trailing whitespace are removed.
  *  - leading space & tabs are converted to the appropriate format.
+ *
+ * All the tokens are inserted before ref. If ref is NULL, they are inserted
+ * at the end of the list.  Line numbers are relative to the start of the data.
  */
-void tokenize(const char *data, int data_len)
+void tokenize(const char *data, int data_len, chunk_t *ref)
 {
    int                idx = 0;
    chunk_t            chunk;
@@ -848,7 +862,7 @@ void tokenize(const char *data, int data_len)
       }
       if (rprev != NULL)
       {
-         pc->flags = rprev->flags & PCF_COPY_FLAGS;
+         pc->flags |= rprev->flags & PCF_COPY_FLAGS;
 
          /* a newline can't be in a preprocessor */
          if (pc->type == CT_NEWLINE)
@@ -856,7 +870,15 @@ void tokenize(const char *data, int data_len)
             pc->flags &= ~PCF_IN_PREPROC;
          }
       }
-      pc = chunk_add(&chunk);
+      if (ref != NULL)
+      {
+         chunk.flags |= PCF_INSERTED;
+      }
+      else
+      {
+         chunk.flags &= ~PCF_INSERTED;
+      }
+      pc = chunk_add_before(&chunk, ref);
 
       /* A newline marks the end of a preprocessor */
       if ((pc->type == CT_NEWLINE) || (pc->type == CT_COMMENT_MULTI))
