@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify.cpp 705 2007-03-04 22:16:24Z bengardner $
+ * $Id: uncrustify.cpp 979 2007-10-25 02:11:52Z bengardner $
  */
 #define DEFINE_PCF_NAMES
 #define DEFINE_CHAR_TABLE
@@ -45,8 +45,10 @@ static char *read_stdin(int& out_len);
 static void uncrustify_file(const char *data, int data_len, FILE *pfout,
                             const char *parsed_file);
 static void do_source_file(const char *filename, FILE *pfout, const char *parsed_file,
-                           const char *prefix, const char *suffix);
-static void process_source_list(const char *source_list, const char *prefix, const char *suffix);
+                           const char *prefix, const char *suffix, bool no_backup);
+static void process_source_list(const char *source_list, const char *prefix,
+                                const char *suffix, bool no_backup);
+static int load_header_files();
 
 /**
  * Replace the brain-dead and non-portable basename().
@@ -75,7 +77,7 @@ const char *path_basename(const char *path)
    while ((ch = *path) != 0)
    {
       path++;
-      if (ch == '/')
+      if (ch == PATH_SEP)
       {
          last_path = path;
       }
@@ -83,6 +85,18 @@ const char *path_basename(const char *path)
    return(last_path);
 }
 
+/**
+ * Returns the length of the directory part of the filename.
+ */
+int path_dirname_len(const char *filename)
+{
+   if (filename == NULL)
+   {
+      return(0);
+   }
+   return(path_basename(filename) - filename);
+}
+
 static void usage_exit(const char *msg, const char *argv0, int code)
 {
    if (msg != NULL)
@@ -107,6 +121,7 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " --suffix SFX : Append SFX to the output filename. The default is '.uncrustify'\n"
            " --prefix PFX : Prepend PFX to the output filename path.\n"
            " --replace    : replace source files (creates a backup)\n"
+           " --no-backup  : replace files, no backup. Useful if files are under source control\n"
            " -l           : language override: C, CPP, D, CS, JAVA, PAWN\n"
            " -t           : load a file with types (usually not needed)\n"
            " -q           : quiet mode - no output on stderr (-L will override)\n"
@@ -123,7 +138,7 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " -p FILE      : dump debug info to a file\n"
            " -L SEV       : Set the log severity (see log_levels.h)\n"
            " -s           : Show the log severity in the logs\n"
-           " --decode FLAG: Print FLAG as text and exit\n"
+           " --decode FLAG: Print FLAG (chunk flags) as text and exit\n"
            "\n"
            "If no input files are specified, the input is read from stdin\n"
            "If -F is used or files are specified on the command line, the output is 'prefix/filename' + suffix\n"
@@ -141,8 +156,8 @@ static void version_exit(void)
 
 int main(int argc, char *argv[])
 {
-   char       *data = NULL;
-   int        data_len = 0;
+   char       *data        = NULL;
+   int        data_len     = 0;
    const char *cfg_file    = "uncrustify.cfg";
    const char *parsed_file = NULL;
    const char *source_file = NULL;
@@ -152,9 +167,10 @@ int main(int argc, char *argv[])
    int        idx;
    const char *p_arg;
 
-   /* If ran without options... show the usage info */
+   /* If ran without options... check keywork sort and show the usage info */
    if (argc == 1)
    {
+      keywords_are_sorted();
       usage_exit(NULL, argv[0], EXIT_SUCCESS);
    }
 
@@ -283,7 +299,8 @@ int main(int argc, char *argv[])
    const char *prefix = arg.Param("--prefix");
    const char *suffix = arg.Param("--suffix");
 
-   if (arg.Present("--replace"))
+   bool no_backup = arg.Present("--no-backup");
+   if (arg.Present("--replace") || no_backup)
    {
       if ((prefix != NULL) || (suffix != NULL))
       {
@@ -395,6 +412,9 @@ int main(int argc, char *argv[])
       return(0);
    }
 
+   /* This relies on cpd.filename being the config file name */
+   load_header_files();
+
    if ((source_file == NULL) && (source_list == NULL) && (p_arg == NULL))
    {
       /* no input specified, so use stdin */
@@ -419,12 +439,12 @@ int main(int argc, char *argv[])
       uncrustify_file(data, data_len, stdout, parsed_file);
       free(data);
       data_len = 0;
-      data = NULL;
+      data     = NULL;
    }
    else if (source_file != NULL)
    {
       /* Doing a single file, output to stdout */
-      do_source_file(source_file, stdout, parsed_file, NULL, NULL);
+      do_source_file(source_file, stdout, parsed_file, NULL, NULL, no_backup);
    }
    else
    {
@@ -442,12 +462,12 @@ int main(int argc, char *argv[])
       idx = 1;
       while ((p_arg = arg.Unused(idx)) != NULL)
       {
-         do_source_file(p_arg, NULL, NULL, prefix, suffix);
+         do_source_file(p_arg, NULL, NULL, prefix, suffix, no_backup);
       }
 
       if (source_list != NULL)
       {
-         process_source_list(source_list, prefix, suffix);
+         process_source_list(source_list, prefix, suffix, no_backup);
       }
    }
 
@@ -457,9 +477,9 @@ int main(int argc, char *argv[])
    return((cpd.error_count != 0) ? 1 : 0);
 }
 
-
 static void process_source_list(const char *source_list,
-                                const char *prefix, const char *suffix)
+                                const char *prefix, const char *suffix,
+                                bool no_backup)
 {
    FILE *p_file = fopen(source_list, "r");
 
@@ -491,12 +511,11 @@ static void process_source_list(const char *source_list,
 
       if ((argc == 1) && (*args[0] != '#'))
       {
-         do_source_file(args[0], NULL, NULL, prefix, suffix);
+         do_source_file(args[0], NULL, NULL, prefix, suffix, no_backup);
       }
    }
 }
 
-
 static char *read_stdin(int& out_len)
 {
    char *data;
@@ -572,6 +591,108 @@ static void make_folders(char *outname)
    }
 }
 
+/**
+ * Loads a file into memory
+ */
+int load_mem_file(const char *filename, file_mem& fm)
+{
+   int         retval = -1;
+   struct stat my_stat;
+   FILE        *p_file;
+
+   fm.data   = NULL;
+   fm.length = 0;
+
+   /* Try to read in the file */
+   p_file = fopen(filename, "rb");
+   if (p_file == NULL)
+   {
+      return(-1);
+   }
+
+   /* this really can't fail... */
+   (void)fstat(fileno(p_file), &my_stat);
+
+   fm.length = my_stat.st_size;
+   fm.data   = (char *)malloc(fm.length + 1);
+   if (fm.data == NULL)
+   {
+      LOG_FMT(LERR, "%s: fopen(%s) failed: out of memory\n", __func__, filename);
+      cpd.error_count++;
+   }
+   else if (fread(fm.data, fm.length, 1, p_file) != 1)
+   {
+      LOG_FMT(LERR, "%s: fread(%s) failed: %s (%d)\n",
+              __func__, filename, strerror(errno), errno);
+      cpd.error_count++;
+   }
+   else
+   {
+      fm.data[fm.length] = 0;
+      retval = 0;
+   }
+   fclose(p_file);
+
+   if ((retval != 0) && (fm.data != NULL))
+   {
+      free(fm.data);
+      fm.data   = NULL;
+      fm.length = 0;
+   }
+   return(retval);
+}
+
+
+/**
+ * Try to load the file from the config folder first and then by name
+ */
+static int load_mem_file_config(const char *filename, file_mem& fm)
+{
+   int  retval;
+   char buf[1024];
+
+   snprintf(buf, sizeof(buf), "%.*s%s",
+            path_dirname_len(cpd.filename), cpd.filename, filename);
+
+   retval = load_mem_file(buf, fm);
+   if (retval < 0)
+   {
+      retval = load_mem_file(filename, fm);
+      if (retval < 0)
+      {
+         LOG_FMT(LERR, "Failed to load (%s) or (%s)\n", buf, filename);
+         cpd.error_count++;
+      }
+   }
+   return(retval);
+}
+
+
+static int load_header_files()
+{
+   int retval = 0;
+
+   if ((cpd.settings[UO_cmt_insert_file_header].str != NULL) &&
+       (cpd.settings[UO_cmt_insert_file_header].str[0] != 0))
+   {
+      retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_file_header].str,
+                                     cpd.file_hdr);
+   }
+   if ((cpd.settings[UO_cmt_insert_func_header].str != NULL) &&
+       (cpd.settings[UO_cmt_insert_func_header].str[0] != 0))
+   {
+      retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_func_header].str,
+                                     cpd.func_hdr);
+   }
+   if ((cpd.settings[UO_cmt_insert_class_header].str != NULL) &&
+       (cpd.settings[UO_cmt_insert_class_header].str[0] != 0))
+   {
+      retval |= load_mem_file_config(cpd.settings[UO_cmt_insert_class_header].str,
+                                     cpd.class_hdr);
+   }
+   return(retval);
+}
+
 /**
  * Does a source file.
  * If pfout is NULL, the source fileaname + ".uncrustify" is used.
@@ -580,14 +701,11 @@ static void make_folders(char *outname)
  * @param pfout    NULL or the output stream
  */
 static void do_source_file(const char *filename, FILE *pfout, const char *parsed_file,
-                           const char *prefix, const char *suffix)
+                           const char *prefix, const char *suffix, bool no_backup)
 {
-   int         data_len;
-   char        *data;
-   bool        did_open = false;
-   FILE        *p_file;
-   struct stat my_stat;
-   bool        need_backup = false;
+   bool     did_open    = false;
+   bool     need_backup = false;
+   file_mem fm;
 
    /* Do some simple language detection based on the filename */
    if (cpd.lang_flags == 0)
@@ -596,25 +714,13 @@ static void do_source_file(const char *filename, FILE *pfout, const char *parsed
    }
 
    /* Try to read in the source file */
-   p_file = fopen(filename, "rb");
-   if (p_file == NULL)
+   if (load_mem_file(filename, fm) < 0)
    {
-      LOG_FMT(LERR, "%s: fopen(%s) failed: %s (%d)\n",
-              __func__, filename, strerror(errno), errno);
+      LOG_FMT(LERR, "Failed to load (%s)\n", filename);
       cpd.error_count++;
       return;
    }
 
-   /*note: could have also just MMAP'd the file, if supported */
-   fstat(fileno(p_file), &my_stat);
-
-   data_len = my_stat.st_size;
-   data     = (char *)malloc(data_len + 1);
-   assert(data != NULL);
-   fread(data, data_len, 1, p_file);
-   data[data_len] = 0;
-   fclose(p_file);
-
    LOG_FMT(LSYS, "Parsing: %s as language %s\n",
            filename, language_to_string(cpd.lang_flags));
 
@@ -623,13 +729,13 @@ static void do_source_file(const char *filename, FILE *pfout, const char *parsed
       char outname[1024];
       int  len = 0;
 
-      if ((prefix == NULL) && (suffix == NULL))
+      if (!no_backup && (prefix == NULL) && (suffix == NULL))
       {
-         if (backup_copy_file(filename, data, data_len) != SUCCESS)
+         if (backup_copy_file(filename, fm.data, fm.length) != SUCCESS)
          {
             LOG_FMT(LERR, "%s: Failed to create backup file for %s\n",
                     __func__, filename);
-            free(data);
+            free(fm.data);
             return;
          }
          need_backup = true;
@@ -651,7 +757,7 @@ static void do_source_file(const char *filename, FILE *pfout, const char *parsed
          LOG_FMT(LERR, "%s: Unable to create %s: %s (%d)\n",
                  __func__, outname, strerror(errno), errno);
          cpd.error_count++;
-         free(data);
+         free(fm.data);
          return;
       }
       did_open = true;
@@ -659,21 +765,66 @@ static void do_source_file(const char *filename, FILE *pfout, const char *parsed
    }
 
    cpd.filename = filename;
-   uncrustify_file(data, data_len, pfout, parsed_file);
+   uncrustify_file(fm.data, fm.length, pfout, parsed_file);
 
-   free(data);
+   free(fm.data);
 
    if (did_open)
    {
       fclose(pfout);
 
-      if (need_backup)
+      if (need_backup && !no_backup)
       {
          backup_create_md5_file(filename);
       }
    }
 }
 
+static void add_file_header()
+{
+   if (!chunk_is_comment(chunk_get_head()))
+   {
+      /*TODO: detect the typical #ifndef FOO / #define FOO sequence */
+      tokenize(cpd.file_hdr.data, cpd.file_hdr.length, chunk_get_head());
+   }
+}
+
+static void add_func_header(c_token_t type, file_mem& fm)
+{
+   chunk_t *pc;
+   chunk_t *ref;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnlnp(pc))
+   {
+      if (pc->type != type)
+      {
+         continue;
+      }
+
+      /* On a function proto or def. Back up to a close brace or semicolon on
+       * the same level
+       */
+      ref = pc;
+      while ((ref = chunk_get_prev(ref)) != NULL)
+      {
+         if (chunk_is_comment(ref) ||
+             ((ref->level != pc->level) &&
+              (ref->flags & PCF_IN_PREPROC)))
+         {
+            break;
+         }
+
+         if ((ref->level == pc->level) &&
+             ((ref->flags & PCF_IN_PREPROC) ||
+              (ref->type == CT_SEMICOLON) ||
+              (ref->type == CT_BRACE_CLOSE)))
+         {
+            tokenize(fm.data, fm.length, chunk_get_next_nnl(ref));
+            break;
+         }
+      }
+   }
+}
 
 static void uncrustify_file(const char *data, int data_len, FILE *pfout,
                             const char *parsed_file)
@@ -681,7 +832,13 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
    /**
     * Parse the text into chunks
     */
-   tokenize(data, data_len);
+   tokenize(data, data_len, NULL);
+
+   /* Add the file header */
+   if (cpd.file_hdr.data != NULL)
+   {
+      add_file_header();
+   }
 
    /**
     * Change certain token types based on simple sequence.
@@ -711,6 +868,18 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
     */
    fix_symbols();
 
+   /**
+    * Add comments before function defs and classes
+    */
+   if (cpd.func_hdr.data != NULL)
+   {
+      add_func_header(CT_FUNC_DEF, cpd.func_hdr);
+   }
+   if (cpd.class_hdr.data != NULL)
+   {
+      add_func_header(CT_CLASS, cpd.class_hdr);
+   }
+
    /**
     * Look at all colons ':' and mark labels, :? sequences, etc.
     */
@@ -731,18 +900,22 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
     */
    do_blank_lines();
    newlines_cleanup_braces();
-   newlines_insert_blank_lines();
-   if (cpd.settings[UO_nl_ds_struct_enum_cmt].b)
+   if (cpd.settings[UO_nl_after_multiline_comment].b)
    {
-      newlines_double_space_struct_enum_union();
+      newline_after_multiline_comment();
    }
+   newlines_insert_blank_lines();
    if (cpd.settings[UO_nl_squeeze_ifdef].b)
    {
       newlines_squeeze_ifdef();
    }
    if (cpd.settings[UO_pos_bool].tp != TP_IGNORE)
    {
-      newlines_bool_pos();
+      newlines_chunk_pos(CT_BOOL, cpd.settings[UO_pos_bool].tp);
+   }
+   if (cpd.settings[UO_pos_comma].tp != TP_IGNORE)
+   {
+      newlines_chunk_pos(CT_COMMA, cpd.settings[UO_pos_comma].tp);
    }
    newlines_class_colon_pos();
    newlines_eat_start_end();
@@ -758,6 +931,19 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       space_text_balance_nested_parens();
    }
 
+   /* Scrub certain added semicolons */
+   if (((cpd.lang_flags & LANG_PAWN) != 0) &&
+       cpd.settings[UO_mod_pawn_semicolon].b)
+   {
+      pawn_scrub_vsemi();
+   }
+
+   /* Scrub extra semicolons */
+   if (cpd.settings[UO_mod_remove_extra_semicolon].b)
+   {
+      remove_extra_semicolons();
+   }
+
    /**
     * Fix same-line inter-chunk spacing
     */
@@ -774,8 +960,8 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
    /**
     * Indent the text
     */
-   indent_text();
    indent_preproc();
+   indent_text();
 
    /**
     * Aligning everything else and reindent
@@ -831,7 +1017,6 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
    }
 }
 
-
 const char *get_token_name(c_token_t token)
 {
    if ((token >= 0) && (token < (int)ARRAY_SIZE(token_names)) &&
@@ -842,7 +1027,6 @@ const char *get_token_name(c_token_t token)
    return("???");
 }
 
-
 static bool ends_with(const char *filename, const char *tag)
 {
    int len1 = strlen(filename);
@@ -855,7 +1039,6 @@ static bool ends_with(const char *filename, const char *tag)
    return(false);
 }
 
-
 struct file_lang
 {
    const char *ext;
@@ -869,6 +1052,7 @@ struct file_lang languages[] =
    { ".cpp",  "CPP",  LANG_CPP  },
    { ".d",    "D",    LANG_D    },
    { ".cs",   "CS",   LANG_CS   },
+   { ".vala", "VALA", LANG_VALA },
    { ".java", "JAVA", LANG_JAVA },
    { ".pawn", "PAWN", LANG_PAWN },
    { ".p",    "",     LANG_PAWN },
@@ -888,7 +1072,7 @@ struct file_lang languages[] =
  * Set idx = 0 before the first call.
  * Done when returns NULL
  */
-const char *get_file_extension(int &idx)
+const char *get_file_extension(int& idx)
 {
    const char *val = NULL;
 
@@ -900,7 +1084,6 @@ const char *get_file_extension(int &idx)
    return(val);
 }
 
-
 /**
  * Find the language for the file extension
  * Default to C
@@ -922,7 +1105,6 @@ static int language_from_filename(const char *filename)
    return(LANG_C);
 }
 
-
 /**
  * Find the language for the file extension
  *
@@ -943,7 +1125,6 @@ static int language_from_tag(const char *tag)
    return(0);
 }
 
-
 /**
  * Gets the tag text for a language
  *
