@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify_types.h 669 2007-02-20 03:55:58Z bengardner $
+ * $Id: uncrustify_types.h 979 2007-10-25 02:11:52Z bengardner $
  */
 
 #ifndef UNCRUSTIFY_TYPES_H_INCLUDED
@@ -37,8 +37,10 @@ enum brstage_e
    BS_ELSE,      /* expecting 'else' after 'if' */
    BS_ELSEIF,    /* expecting 'if' after 'else' */
    BS_WHILE,     /* expecting 'while' after 'do' */
+   BS_CATCH,     /* expecting 'catch' or 'finally' after 'try' */
 };
 
+struct chunk_t;
 
 /**
  * Structure for counting nested level
@@ -48,6 +50,7 @@ struct paren_stack_entry
    c_token_t type;         /**< the type that opened the entry */
    int       level;        /**< Level of opening type */
    int       open_line;    /**< line that open symbol is on */
+   chunk_t   *pc;          /**< Chunk that opened the level */
    int       brace_indent; /**< indent for braces - may not relate to indent */
    int       indent;       /**< indent level (depends on use) */
    int       indent_tmp;   /**< temporary indent level (depends on use) */
@@ -55,7 +58,7 @@ struct paren_stack_entry
    int       ref;
    c_token_t parent;       /**< if, for, function, etc */
    brstage_e stage;
-   bool      in_preproc;   /**> whether this was created in a preprocessor */
+   bool      in_preproc;   /**< whether this was created in a preprocessor */
 };
 
 /* TODO: put this on a linked list */
@@ -79,11 +82,11 @@ struct parse_frame
    bool                     maybe_cast;
 };
 
-#define PCF_STMT_START         (1 << 0)  /* marks the start of a statment */
-#define PCF_EXPR_START         (1 << 1)
-#define PCF_IN_PREPROC         (1 << 2)  /* in a preprocessor */
-#define PCF_DONT_INDENT        (1 << 3)  /* already aligned! */
-#define PCF_MFC_PAREN          (1 << 4)  /* macro function close paren */
+#define PCF_FORCE_SPACE        (1 << 0)  /* must have a space after this token */
+#define PCF_STMT_START         (1 << 1)  /* marks the start of a statment */
+#define PCF_EXPR_START         (1 << 2)
+#define PCF_IN_PREPROC         (1 << 3)  /* in a preprocessor */
+#define PCF_DONT_INDENT        (1 << 4)  /* already aligned! */
 #define PCF_VAR_DEF            (1 << 5)  /* variable name in a variable def */
 #define PCF_VAR_1ST            (1 << 6)  /* 1st variable def in a statement */
 #define PCF_VAR_1ST_DEF        (PCF_VAR_DEF | PCF_VAR_1ST)
@@ -95,7 +98,7 @@ struct parse_frame
 #define PCF_RIGHT_COMMENT      (1 << 12)
 #define PCF_OLD_FCN_PARAMS     (1 << 13)
 #define PCF_WAS_ALIGNED        (1 << 14)
-#define PCF_OPTIONAL           (1 << 15)
+#define PCF_IN_TEMPLATE        (1 << 15)
 #define PCF_IN_TYPEDEF         (1 << 16)
 #define PCF_IN_CONST_ARGS      (1 << 17)
 #define PCF_LVALUE             (1 << 18) /* left of assignment */
@@ -105,21 +108,30 @@ struct parse_frame
 #define PCF_IN_FOR             (1 << 22)
 #define PCF_ONE_LINER          (1 << 23)
 #define PCF_ONE_CLASS          (PCF_ONE_LINER | PCF_IN_CLASS)
+#define PCF_EMPTY_BODY         (1 << 24)
+#define PCF_ANCHOR             (1 << 25)  /* aligning anchor */
+#define PCF_PUNCTUATOR         (1 << 26)
+#define PCF_INSERTED           (1 << 27)  /* chunk was inserted from another file */
+#define PCF_TBD_28             (1 << 28)
+#define PCF_TBD_29             (1 << 29)
+#define PCF_TBD_30             (1 << 30)
+#define PCF_TBD_31             (1 << 31)
+
 
 /* flags that get copied when a new chunk is inserted */
-#define PCF_COPY_FLAGS         (PCF_IN_PREPROC | PCF_IN_SPAREN | PCF_IN_ENUM |      \
-                                PCF_IN_FCN_DEF | PCF_IN_FCN_CALL | PCF_IN_TYPEDEF | \
+#define PCF_COPY_FLAGS         (PCF_IN_PREPROC | PCF_IN_SPAREN | PCF_IN_ENUM |          \
+                                PCF_IN_FCN_DEF | PCF_IN_FCN_CALL | PCF_IN_TYPEDEF |     \
                                 PCF_IN_ARRAY_ASSIGN | PCF_IN_CLASS | PCF_IN_NAMESPACE | \
-                                PCF_IN_CLASS | PCF_IN_FOR)
+                                PCF_IN_CLASS | PCF_IN_FOR | PCF_IN_TEMPLATE)
 
 #ifdef DEFINE_PCF_NAMES
 static const char *pcf_names[] =
 {
+   "FORCE_SPACE",
    "STMT_START",
    "EXPR_START",
    "IN_PREPROC",
    "DONT_INDENT",
-   "MFC_PAREN",
    "VAR_DEF",
    "VAR_1ST",
    "VAR_INLINE",
@@ -130,7 +142,7 @@ static const char *pcf_names[] =
    "RIGHT_COMMENT",
    "OLD_FCN_PARAMS",
    "WAS_ALIGNED",
-   "OPTIONAL",
+   "IN_TEMPLATE",
    "IN_TYPEDEF",
    "IN_CONST_ARGS",
    "LVALUE",
@@ -139,6 +151,14 @@ static const char *pcf_names[] =
    "IN_NAMESPACE",
    "IN_FOR",
    "ONE_LINER",
+   "EMPTY_BODY",
+   "ANCHOR",
+   "PUNCTUATOR",
+   "INSERTED",
+   "TBD_28",
+   "TBD_29",
+   "TBD_30",
+   "TBD_31",
 };
 #endif
 
@@ -167,18 +187,19 @@ struct chunk_t
 
 enum
 {
-   LANG_C    = 0x01,
-   LANG_CPP  = 0x02,
-   LANG_D    = 0x04,
-   LANG_CS   = 0x08,     /*<< C# or C-sharp */
-   LANG_JAVA = 0x10,
-   LANG_OC   = 0x20,     /*<< Objective C */
-   LANG_PAWN = 0x40,
+   LANG_C    = 0x0001,
+   LANG_CPP  = 0x0002,
+   LANG_D    = 0x0004,
+   LANG_CS   = 0x0008,     /*<< C# or C-sharp */
+   LANG_JAVA = 0x0010,
+   LANG_OC   = 0x0020,     /*<< Objective C */
+   LANG_VALA = 0x0040,     /*<< Like C# */
+   LANG_PAWN = 0x0080,
 
-   LANG_ALLC = 0x3f,
-   LANG_ALL  = 0x7f,
+   LANG_ALLC = 0x007f,
+   LANG_ALL  = 0x00ff,
 
-   FLAG_PP   = 0x80,     /*<< only appears in a preprocessor */
+   FLAG_PP   = 0x8000,     /*<< only appears in a preprocessor */
 };
 
 /**
@@ -187,18 +208,19 @@ enum
 enum pattern_class
 {
    PATCLS_NONE,
-   PATCLS_BRACED,   // keyword + braced stmt:          else, do, try
+   PATCLS_BRACED,   // keyword + braced stmt:          do, try
    PATCLS_PBRACED,  // keyword + parens + braced stmt: switch, if, for, while
    PATCLS_OPBRACED, // keyword + optional parens + braced stmt: catch, version
    PATCLS_VBRACED,  // keyword + value + braced stmt: namespace
    PATCLS_PAREN,    // keyword + parens: while-of-do
+   PATCLS_ELSE,     // Special case of PATCLS_BRACED for handling CT_IF
 };
 
 typedef struct
 {
    const char *tag;
    c_token_t  type;
-   UINT8      lang_flags;
+   int        lang_flags;
 } chunk_tag_t;
 
 typedef struct
@@ -236,6 +258,12 @@ typedef struct chunk_stack
    int                 size;
 } chunk_stack_t;
 
+struct file_mem
+{
+   char *data;
+   int  length;
+};
+
 struct cp_data
 {
    FILE               *fout;
@@ -243,7 +271,11 @@ struct cp_data
    UINT32             error_count;
    const char         *filename;
 
-   UINT8              lang_flags; // LANG_xxx
+   file_mem           file_hdr;  /* for cmt_insert_file_header */
+   file_mem           func_hdr;  /* for cmt_insert_func_header */
+   file_mem           class_hdr; /* for cmt_insert_class_header */
+
+   int                lang_flags; // LANG_xxx
 
    UINT16             line_number;
    UINT16             column;  /* column for parsing */
@@ -261,9 +293,6 @@ struct cp_data
    /* bumped up when a line is split or indented */
    int                changes;
 
-   /* dummy entries */
-   chunk_t            list_chunks;
-
    struct align_t     al[80];
    int                al_cnt;
    bool               al_c99_array;
