@@ -2,7 +2,7 @@
  * @file universalindentgui.cpp
  * Exports the config file for UniversalIndentGUI
  *
- * $Id: universalindentgui.cpp 705 2007-03-04 22:16:24Z bengardner $
+ * $Id: universalindentgui.cpp 964 2007-10-15 00:10:13Z bengardner $
  */
 #include "prototypes.h"
 #include <stdio.h>
@@ -39,7 +39,7 @@ void print_universal_indent_cfg(FILE *pfile)
 
 
    /* Add all the recognized file extensions */
-   ch = '=';
+   ch  = '=';
    idx = 0;
    fprintf(pfile, "fileTypes");
    while ((p_name = get_file_extension(idx)) != NULL)
@@ -58,6 +58,7 @@ void print_universal_indent_cfg(FILE *pfile)
            "outputFileName=indentoutput\n"
            "outputFileParameter=\"-o \"\n"
            "stringparaminquotes=false\n"
+           "parameterOrder=ipo\n"
            "useCfgFileParameter=\"-c \"\n");
 
    /* Now add each option */
@@ -185,6 +186,13 @@ void print_universal_indent_cfg(FILE *pfile)
                     option->name, option->name, option->name);
             break;
 
+         case AT_STRING:
+            // TODO: figure out what is needed for string options
+            // fprintf(pfile, "EditorType=multiple\n");
+            // fprintf(pfile, "Choices=%s=ignore|%s=lead|%s=trail\n",
+            //         option->name, option->name, option->name);
+            break;
+
          default:
             break;
          }
