@@ -22,15 +22,15 @@ static_inline bool is_past_width(chunk_t *pc)
    return((pc->column + pc->len) > cpd.settings[UO_code_width].n);
 }
 
-
 /**
  * Split right after the chunk
  */
 static void split_before_chunk(chunk_t *pc)
 {
-   LOG_FMT(LSYS, "%s: %.*s\n", __func__, pc->len, pc->str);
+   LOG_FMT(LSPLIT, "%s: %.*s\n", __func__, pc->len, pc->str);
 
-   if (!chunk_is_newline(chunk_get_prev(pc)))
+   if (!chunk_is_newline(pc) &&
+       !chunk_is_newline(chunk_get_prev(pc)))
    {
       newline_add_before(pc);
       reindent_line(pc, pc->brace_level * cpd.settings[UO_indent_columns].n);
@@ -38,7 +38,6 @@ static void split_before_chunk(chunk_t *pc)
    }
 }
 
-
 /**
  * Step forward until a token goes beyond the limit and then call split_line()
  * to split the line at or before that point.
@@ -51,7 +50,10 @@ void do_code_width(void)
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
-      if (!chunk_is_newline(pc) && !chunk_is_comment(pc) && is_past_width(pc))
+      if (!chunk_is_newline(pc) &&
+          !chunk_is_comment(pc) &&
+          (pc->type != CT_SPACE) &&
+          is_past_width(pc))
       {
          split_line(pc);
       }
@@ -77,8 +79,8 @@ static const token_pri pri_table[] =
    { CT_BOOL,      3 },
    { CT_COMPARE,   4 },
    { CT_ARITH,     5 },
-   { CT_DC_MEMBER, 10 },
-   { CT_MEMBER,    10 },
+   //{ CT_DC_MEMBER, 10 },
+   //{ CT_MEMBER,    10 },
 };
 
 
@@ -96,7 +98,6 @@ static int get_split_pri(c_token_t tok)
    return(0);
 }
 
-
 /**
  * Checks to see if pc is a better spot to split.
  * This should only be called going BACKWARDS (ie prev)
@@ -148,7 +149,6 @@ static void try_split_here(cw_entry& ent, chunk_t *pc)
    }
 }
 
-
 /**
  * Scan backwards to find the most appropriate spot to split the line
  * and insert a newline.
@@ -160,9 +160,10 @@ static void try_split_here(cw_entry& ent, chunk_t *pc)
  */
 static void split_line(chunk_t *start)
 {
-   LOG_FMT(LSPLIT, "%s: line %d, col %d token:%.*s (IN_FUNC=%d) ",
+   LOG_FMT(LSPLIT, "%s: line %d, col %d token:%.*s[%s] (IN_FUNC=%d) ",
            __func__, start->orig_line, start->column, start->len, start->str,
-           (start->flags & PCF_IN_FCN_DEF) != 0);
+           get_token_name(start->type),
+           (start->flags & (PCF_IN_FCN_DEF | PCF_IN_FCN_CALL)) != 0);
 
    /* Don't break before a close, comma, or colon */
    if ((start->type == CT_PAREN_CLOSE) ||
@@ -200,7 +201,7 @@ static void split_line(chunk_t *start)
     * after the open paren
     */
    else if (((start->flags & PCF_IN_FCN_DEF) != 0) ||
-            ((start->level == start->brace_level) &&
+            ((start->level == (start->brace_level + 1)) &&
              ((start->flags & PCF_IN_FCN_CALL) != 0)))
    {
       LOG_FMT(LSPLIT, " ** FUNC SPLIT **\n");
@@ -232,17 +233,21 @@ static void split_line(chunk_t *start)
 
    while (((pc = chunk_get_prev(pc)) != NULL) && !chunk_is_newline(pc))
    {
-      try_split_here(ent, pc);
+      if (pc->type != CT_SPACE)
+      {
+         try_split_here(ent, pc);
+      }
    }
 
    if (ent.pc == NULL)
    {
-      LOG_FMT(LSYS, "%s: TRY_SPLIT yeilded NO SOLUTION for line %d at %.*s\n",
-              __func__, start->orig_line, start->len, start->str);
+      LOG_FMT(LSPLIT, "%s: TRY_SPLIT yielded NO SOLUTION for line %d at %.*s [%s]\n",
+              __func__, start->orig_line, start->len, start->str, get_token_name(start->type));
    }
    else
    {
-      LOG_FMT(LSYS, "%s: TRY_SPLIT yeilded %.*s\n", __func__, ent.pc->len, ent.pc->str);
+      LOG_FMT(LSPLIT, "%s: TRY_SPLIT yielded '%.*s' [%s] on line %d\n", __func__,
+              ent.pc->len, ent.pc->str, get_token_name(ent.pc->type), ent.pc->orig_line);
    }
 
    pc = chunk_get_next(ent.pc);
@@ -265,7 +270,6 @@ static void split_line(chunk_t *start)
    }
 }
 
-
 /**
  * A for statment is too long.
  * Step backwards and forwards to find the semicolons
@@ -277,7 +281,7 @@ static void split_for_stmt(chunk_t *start)
    chunk_t *st[2];
    chunk_t *pc = start;
 
-   LOG_FMT(LSYS, "\n\n%s: starting on %.*s\n", __func__, pc->len, pc->str);
+   LOG_FMT(LSPLIT, "%s: starting on %.*s\n", __func__, pc->len, pc->str);
 
    /* see if we started on the semicolon */
    if ((pc->type == CT_SEMICOLON) && (pc->parent_type == CT_FOR))
@@ -308,12 +312,11 @@ static void split_for_stmt(chunk_t *start)
 
    while (--count >= 0)
    {
-      LOG_FMT(LSYS, "%s: %.*s\n", __func__, st[count]->len, st[count]->str);
+      LOG_FMT(LSPLIT, "%s: %.*s\n", __func__, st[count]->len, st[count]->str);
       split_before_chunk(chunk_get_next(st[count]));
    }
 }
 
-
 /**
  * Splits the parameters at every comma that is at the fparen level.
  *
@@ -321,7 +324,7 @@ static void split_for_stmt(chunk_t *start)
  */
 static void split_fcn_params_full(chunk_t *start)
 {
-   LOG_FMT(LSPLIT, "\n\n  %s: ", __func__);
+   LOG_FMT(LSPLIT, "%s", __func__);
 
    chunk_t *fpo;
    chunk_t *pc;
@@ -349,7 +352,6 @@ static void split_fcn_params_full(chunk_t *start)
    }
 }
 
-
 /**
  * Figures out where to split a function def/proto/call
  *
