@@ -1,3 +1,8 @@
+struct bar;
+struct foo {
+  operator bar*();
+};
+
 class Foo {
 Foo operator+(const Foo& rhs) const;
 
@@ -24,8 +29,13 @@ class Example
 {
 char m_array [256];
 
+operator Foo::Bar();
+operator Foo::Bar*();
+operator Foo::Bar&();
+
 int operator()(int index)
 {
+  i = ~~3;
   return index + 1;
 }
 
