@@ -1,3 +1,10 @@
+int &
+Function()
+{
+   static int x;
+   return (x);
+}
+
 void
 foo1(int param1, int param2, char *param2);
 
@@ -24,9 +31,11 @@ foo(int param1, int param2, char *param2)
    printf("boo!\n");
 }
 
-int classname::method();
+int
+classname::method();
 
-int classname::method()
+int
+classname::method()
 {
    foo();
 }
@@ -52,3 +61,15 @@ className::method2(void) const
    // stuff
 }
 
+int &
+foo();
+
+int &
+foo()
+{
+   list_for_each(a,b) {
+      bar(a);
+   }
+   return nuts;
+}
+
