@@ -6,7 +6,7 @@
 # This could all be done with bash, but I wanted to use python. =)
 # Anyway, this was all done while waiting in the Denver airport.
 #
-# $Id: run_tests.py 687 2007-02-24 21:27:02Z bengardner $
+# $Id: run_tests.py 948 2007-10-01 17:48:40Z bengardner $
 #
 
 import sys
@@ -150,7 +150,7 @@ if __name__ == '__main__':
 			args.append(arg)
 
 	if len(args) == 0:
-		the_tests += "c-sharp c cpp d java pawn objective-c".split()
+		the_tests += "c-sharp c cpp d java pawn objective-c vala".split()
 	else:
 		the_tests += args
 
