@@ -6,7 +6,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: windows_compat.h 621 2007-01-19 02:33:56Z bengardner $
+ * $Id: windows_compat.h 923 2007-09-18 13:04:22Z bengardner $
  */
 #ifndef WINDOWS_COMPAT_H_INCLUDED
 #define WINDOWS_COMPAT_H_INCLUDED
@@ -53,6 +53,9 @@ typedef unsigned int       UINT32;
 #undef strcasecmp
 #define strcasecmp    _strcmpi
 
+#undef strncasecmp
+#define strncasecmp   _strnicmp
+
 /* includes for _setmode() */
 #include <io.h>
 #include <fcntl.h>
@@ -61,4 +64,5 @@ typedef unsigned int       UINT32;
 #define mkdir(x, y) _mkdir(x)
 #define PATH_SEP  '\\'
 
-#endif   /* WINDOWS_COMPAT_H_INCLUDED */
\ No newline at end of file
+#endif   /* WINDOWS_COMPAT_H_INCLUDED */
+
